package remote;

import sopra.Order;
import java.util.Observable;
import java.util.Stack;
import java.util.Vector;
import java.util.Date;

class InputDeviceEngine extends Observable {
	/** 
	 * Interner Zustand des InputDevices. 
	 * state = 0 -> Initialzustand, Eingabe der AgentID <BR>
	 * state = 1 -> Vergabe des Zeitstempels fuer den Order, System <BR>
	 * state = 2 -> Origin, Eingabe Vorkommastellen von Origin <BR>
	 * state = 3 -> Origin, Eingabe Nachkommastellen (von Origin) Point eingegeben <BR>
	 * state = 4 -> Origin, Eingabe Nachkommastellen von Origin <BR>
	 * state = 5 -> OrderItems, Eingabe der ArtikelID <BR>
	 * state = 6 -> OrderItems, Eingabe der Quantity <BR>
	 * state = 7 -> Transfer durchgefuehrt <BR>
	*/

	private Vector orderVector;
	/** speichert alle Orders bis zum Transfer hier*/

	private Order order; /** einzelnes Order-Objekt*/

	/** speichert die OrderItems in einem Vector temporaer ab*/
	private Vector orderItemsVector;
	
	private int agent; /** muss gespeichert werden, da nur einmal eingegeben*/
	private int state;
	/** speichert den jeweiligen State des Zustandsautomaten InputDeviceEngine*/
	/** Stack fuer die Operanden. Der oberste Wert im Operandenstack repraesentiert
	den Akku der InputDevice. */
	private Stack ops;

	private Op op; /** Durchzufuehrende Operation */

	/** Beinhaltet zu den Zustaenden die jeweilige textuelle Beschreibung*/
	private final static String[] stateString =
		{
			"Initialize the inputDevice by entering your agentID:",
			"ready for orders, press button 'order'",
			"(origin) table.customer: enter table",
			"OriginENDE never visible: Gast",
			"(origin) table.customer: enter customer",
			"(orderitems) articleID, quantity: enter artikelID",
			"(orderitems) articleID, quantity: enter quantity",
			"transfer successfully done" };

	InputDeviceEngine() {
		this.clear();
		state = 0;
		this.orderVector = new Vector();
		notifyObservers();
	}

	Stack ops() {
		return this.ops;
	}

	void ops(Stack aStack) {
		this.ops = aStack;
	}

	Op op() {
		return this.op;
	}

	private String state() {
		return stateString[this.state];
	}

	void op(Op anOp) {
		this.op = anOp;
	}

	private Order order() {
		return this.order;
	}

	private void order(Order anOrder) {
		this.order = anOrder;
	}

	private Vector orderVector() {
		return this.orderVector;
	}

	private Vector orderItemsVector() {
		return this.orderItemsVector;
	}

	private int agent() {
		return this.agent;
	}

	private void agent(int anAgentID) {
		this.agent = anAgentID;
	}

	private boolean isZeroValue() {

		if (ops().empty()) {
			return false;
		}

		String temp;

		// Aktuellen Wert des Akkus vom Stack holen und in String umwandeln
		temp = (String) ops().peek();

		if (((state == 0)
			|| (state == 2)
			|| (state == 4)
			|| (state == 5)
			|| (state == 6))
			&& (!((temp.equalsIgnoreCase("00") || temp.equalsIgnoreCase("0"))))) {
			return false;
			// Keine 0-Werte alleine zulassenwieder zurueckschreiben
		}
		return true;
	}

	private void createElement() {
		try {
			this.order((Order) Order.class.newInstance());
			this.orderItemsVector = new Vector();
		} catch (IllegalAccessException e) {
			System.out.println(e);
		} catch (InstantiationException e) {
			System.out.println(e);
		}
	}

	private void addToOrders(Order anOrder) {
		this.orderVector().addElement(anOrder);
	}

	private void addToOrderItems(Object anOrderItemPart) {
		this.orderItemsVector().addElement(anOrderItemPart);
	}

	private void finishOrder() {
		this.addToOrderItems(ops.pop().toString());
		//Elemente in orderItems setzen
		this.order().setOrderItems(orderItemsVectorToArray());
		// OrderItems in Order speichern
		this.addToOrders(this.order()); //aktuellen Order in Vector speichern
	}
	
	private void initializeOrder() {
		this.createElement();
		this.order().setAgent(this.agent()); // setzen des Agents
		this.order().setOrderTime(new Date()); // setzen der Systemzeit
	}
	
	public int[] orderItemsVectorToArray() {
		if (this.orderItemsVector().size() % 2 == 1) {
			System.out.println(
				"ERROR in Anzahl der OrderItems: Alles wird zurueck gesetzt");
			this.clear();
		}
		int[] tmpOrderItems = new int[this.orderItemsVector().size()];
		for (int i = 0; i < this.orderItemsVector().size(); i++) {
			tmpOrderItems[i] =
				convertStringToInt(
					this.orderItemsVector().elementAt(i).toString());
		}
		this.orderItemsVector().removeAllElements();
		return tmpOrderItems;
	}
	
	public void notifyObservers() {
		setChanged();
		notifyObservers(new Op(ops().peek().toString(), this.state()));
	}

	synchronized void period() {
		String temp;
		if ((!(isZeroValue()))
			&& (state == 2)) {
			// Wenn Zustand "Origin, Eingabe Vorkommastellen von Origin ", dann
			// neuer Zustand "Origin, Eingabe Nachkommastellen (von Origin) gewuenscht (Point)"
			temp = (String) ops().pop();
			temp = temp + ".";
			// Wichtig: geht hier direkt auf State 4 ueber, da State 3 akzeptiert
			state = 4;
			ops.push(temp); // neuer Akku-Wert wieder auf Stack
			notifyObservers();
			// Es hat sich was veraendert -> Observer benachrichtigen
		}
	}

	synchronized void digit(String aString) {
		String temp;
		// Wenn zuvor "Transfer" gedrueckt wurde, dann vor Eingabe einer
		// neuen Ziffer den Rechner initialisieren
		if (state == 7) {
			clear();
		};

		// Aktuellen Wert des Akkus vom Stack holen und in String umwandeln
		temp = (String) ops().pop();

		if (state == 1) {
			// Gelangt hier hin nach 'Cancel', tue nichts in diesem Status
			ops().push("0");
		} else { // Ansonsten Ziffer an den alten Wert des Akkus anhaengen und Zust. beibehalten
			temp = temp + aString;
			// Abbruch, wenn Status nicht erlaubt (0-Werte)
			if (isZeroValue()) {
				return;
			}
			// ggf. fuehrende Null eliminieren
			if (temp.startsWith("0")) // zu implementieren
				{
				temp = temp.substring(1);
			}
		};
		ops.push(temp); // neuer Akku-Wert wieder auf Stack
		notifyObservers();
		// Es hat sich was veraendert -> Observer benachrichtigen
	}

	synchronized void enter() {
		if (isZeroValue()) {
			return;
		}
		if (state == 0) { // einmalige initialisierung des Agents
			this.agent(convertStringToInt(ops.peek().toString()));
			ops().push("0");
			state = 1;
		} else if (state == 4) { // Setze Origin des aktuellen Orders
			this.order().setOrigin(convertStringToFloat(ops.peek().toString()));
			ops().push("0");
			state = 5;
		} else if (state == 5) { // lege in orderItems ab (ArticleID)
			this.addToOrderItems(ops.peek());
			ops().push("0");
			state = 6;
		} else if (state == 6) { // lege in orderItems ab (Quantity)
			this.addToOrderItems(ops.peek());
			ops().push("0");
			// Gehe zurueck auf State 5
			state = 5;
		}
		notifyObservers();
	}

	synchronized void clear() {
		// Stack loeschen
		ops(new Stack());
		// Akku initialisieren
		ops().push("0");
		// Operationsspeicher loeschen
		op(null);
		// Bereit
		if (state > 0) {
			state = 1;
		}
		notifyObservers();
	}

	/**	Post:	Transfertaste wurde gedrueckt. Abschliessende Berechnung durchfuehren
					durch Status-Sicherung des aktuellen Order-Objektes und schliesslich Transfer 
					an DockingStation, indem alle gespeicherten Orders im ordersVector an Dockingstation 
					uebergeben werden. OrderVector() neu initialisieren und State (7) setzen.<BR>
		Prec:	-
		...
		
		MUSS IM VERSUCH 8 IMPLEMENTIERT WERDEN
	*/
	synchronized void transfer() {
	};

	synchronized void enterOrder() {
		// nur diese 3 Zustaende werden hier akzeptiert
		if ((state == 1)
			|| ((state == 6) && (!isZeroValue()))
			|| (state == 7)) {
			if (state == 6) {
				this.finishOrder();
			}
			this.initializeOrder();
			this.clear();
			state = 2;
			notifyObservers();
		}
	}
	
	private int convertStringToInt(String valueAsString) {
		try {
			Integer idObject =
				(Integer) util.StringToObjectConverter.Convert(
					valueAsString,
					int.class);
			return idObject.intValue();
		} catch (NumberFormatException e) {
			System.out.println(e);
		}
		return -1;
	}

	private float convertStringToFloat(String valueAsString) {
		try {
			Float idObject =
				(Float) util.StringToObjectConverter.Convert(
					valueAsString,
					float.class);
			return idObject.floatValue();
		} catch (NumberFormatException e) {
			System.out.println(e);
		}
		return -1;
	}
}