package remote;

import java.awt.BorderLayout;
import java.awt.Button;
import java.awt.Color;
import java.awt.GridLayout;
import java.awt.Panel;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import sopra.Order;

class InputDeviceKeypad extends Panel implements ActionListener {

	private Order order;

	/** "InputDevice-Model", welches die Berechnung durchfuehrt */
	InputDeviceEngine ie;

	/** Zahlentasten: b0=0, b1=1 etc. */
	private Button b0, b1, b2, b3, b4, b5, b6, b7, b8, b9;
	private Button bPnt; /** Dezimalpunkt */
	/** Spezial Tasten */
	private Button bClr; /** Clear-Taste */
	private Button bEnt; /** Enter-Taste */
	private Button bOrd; /** New Order-Taste */
	private Button bTra; /** Transfer-Taste */

	InputDeviceKeypad(InputDeviceEngine ie) {
		this.ie = ie; // Eintragen der InputDeviceEngine

		/* Basislayout ist ein Borderlayout, in der Mitte ("Center") die Zahlen
		und Operationstasten. Rechts ("East") angeordnet die Spezialtasten */
		setLayout(new BorderLayout(4, 4));

		/* Setzen der Spezialtasten. Hierzu wird ein Panel erzeugt, welches
		ein GridLayout mit zwei Zeilen enthaelt. Dort hinein kommen die
		beiden Spezialtasten */
		Panel p1 = new Panel();
		p1.setLayout(new GridLayout(2, 2, 4, 4));
		add("East", p1);
		bClr = initAndAddToPanel(new Button("clear"), Color.red, p1);
		bEnt = initAndAddToPanel(new Button("enter"), Color.orange, p1);
		bOrd = initAndAddToPanel(new Button("order"), Color.orange, p1);
		bTra = initAndAddToPanel(new Button("transfer"), Color.green, p1);

		/* Setzen der Zahlen- und Operationstasten. Hierzu wird ein Panel
		erzeugt, welche ein GridLayout mit 4 Zeilen und 4 Spalten enthaelt.
		In dieses Grid kommen die 16 Zahlen und Operationstasten */
		Panel p2 = new Panel();
		p2.setLayout(new GridLayout(4, 4, 4, 4));
		add("Center", p2);

		/* hinzufuegen der Tasten der 1. Zeile des Tastenfeldes */
		b1 = initAndAddToPanel(new Button("1"), Color.white, p2);
		b2 = initAndAddToPanel(new Button("2"), Color.white, p2);
		b3 = initAndAddToPanel(new Button("3"), Color.white, p2);
		/* hinzufuegen der Tasten der 2. Zeile des Tastenfeldes */
		b4 = initAndAddToPanel(new Button("4"), Color.white, p2);
		b5 = initAndAddToPanel(new Button("5"), Color.white, p2);
		b6 = initAndAddToPanel(new Button("6"), Color.white, p2);

		/* hinzufuegen der Tasten der 3. Zeile des Tastenfeldes */
		b7 = initAndAddToPanel(new Button("7"), Color.white, p2);
		b8 = initAndAddToPanel(new Button("8"), Color.white, p2);
		b9 = initAndAddToPanel(new Button("9"), Color.white, p2);

		/* hinzufuegen der Tasten der 4. Zeile des Tastenfeldes */
		b0 = initAndAddToPanel(new Button("0"), Color.white, p2);
		bPnt = initAndAddToPanel(new Button("."), Color.white, p2);
	}

	public void actionPerformed(ActionEvent e) {
		/* Tasten 0 - 9 wurde gedrueckt. Wenn eine solche Taste gedrueckt
		wurde, dann wird das Argument <arg> in einen String uebersetzt und der
		CalculatorEngine <ce> mitgeteilt welche Taste gedrueckt wurde */
		if (e.getSource() == b1) {
			ie.digit("1");
		} else if (e.getSource() == b2) {
			ie.digit("2");
		} else if (e.getSource() == b3) {
			ie.digit("3");
		} else if (e.getSource() == b4) {
			ie.digit("4");
		} else if (e.getSource() == b5) {
			ie.digit("5");
		} else if (e.getSource() == b6) {
			ie.digit("6");
		} else if (e.getSource() == b7) {
			ie.digit("7");
		} else if (e.getSource() == b8) {
			ie.digit("8");
		} else if (e.getSource() == b9) {
			ie.digit("9");
		} else if (e.getSource() == b0) {
			ie.digit("0");
		}
		/* Dezimalpunkt wurde gedrueckt. Entsprechende Nachricht geht an
		die CalculatorEngine <ie> */
		else if (e.getSource() == bPnt) {
			ie.period();
		}
		/* Spezialtaste wurde gedrueckt. Entsprechende Nachricht (clear oder
		calculate) geht an die CalculatorEngine <ie> */
		else if (e.getSource() == bEnt) {
			ie.enter();
		} else if (e.getSource() == bClr) {
			ie.clear();
		} else if (e.getSource() == bOrd) {
			ie.enterOrder();
		} else if (e.getSource() == bTra) {
			ie.transfer();
		}
	}

	private Button initAndAddToPanel(
		Button aButton,
		Color backgroundColor,
		Panel aPanel) {
		aButton.setBackground(backgroundColor);
		aButton.addActionListener(this);
		aPanel.add(aButton);
		return aButton;
	}
}