package remote;

import sopra.Order;

import java.util.Observable;
import java.util.Stack;
import java.util.Vector;
import java.util.Date;

/**-----------------------------------------------------------------------------------
@author		Nancy Schett (NS)
@java			Java v1.1.x - Metroworks CodeWarrior Pro 5, IDE4.04 
@project		Software-Praktikum
@copyright	Forschungsgruppe Requirements Engineering,
				Institut fuer Informatik, Universitaet Zuerich
@class		InputDeviceEngine
@see			Op
@version		1.0	-	NS, 2000-March-29; erste Version
				1.1	- 	NS, 15-April-2000, clear korrigiert, Transfer implementiert.
--------------------------------------------------------------------------------------*/
/**<BR>
Intention: Rechenwerk der InputDevice. In Abhaengigkeit vom jeweiligen Zustand
akzeptiert die InputDevice Zahlen oder Operationseingaben (enter, transfer, cancel, order). 
Hier werden temporaer, bei gestartetem Boersenc.-Betrieb -bedient durch die jeweiligen 
Angestellten (agent)-, die Orders (Bestellungen) gespeichert. Beim Aufstarten des Input
Devices (state==0) wird der Agent einmalig initialisiert. Danach kann mit den Eingaben
der Orders begonnen werden, welche -falls nicht 'cancel' gedrueckt wurde-
bei erfolgreichem Abschluss in einem Vektor 'orderVector' gespeichert werden
und darauf warten, bis der Agngestellt 'transfer' drueckt (temporaer->permanent). Somit
werden alle offenen Bestellungen (im 'orderVector') an die Docking-Station uebergeben und
der orderVector wird neu initialisiert, wie auch die InputDevice.<BR>

Responsabilities: <BR>*/

class InputDeviceEngine extends Observable
	{

	/** Interner Zustand des InputDevices. 
	state = 0 -> Initialzustand, Eingabe der AgentID <BR>
	state = 1 -> Vergabe des Zeitstempels fuer den Order, System <BR>
	state = 2 -> Origin, Eingabe Vorkommastellen von Origin <BR>
	state = 3 -> Origin, Eingabe Nachkommastellen (von Origin) Point eingegeben <BR>
	state = 4 -> Origin, Eingabe Nachkommastellen von Origin <BR>
	state = 5 -> OrderItems, Eingabe der ArtikelID <BR>
	state = 6 -> OrderItems, Eingabe der Quantity <BR>
	state = 7 -> Transfer durchgefuehrt <BR> */
	
	//// Instanzvariables
	private Vector orderVector;			/** speichert alle Orders bis zum Transfer hier*/
	
	private Order order;						/** einzelnes Order-Objekt*/

	private Vector orderItemsVector;		/** speichert die OrderItems in einem Vector temporaer ab*/
	private int agent;						/** muss gespeichert werden, da nur einmal eingegeben*/

	private int state;						/** speichert den jeweiligen State des Zustandsautomaten InputDeviceEngine*/
	/** Stack fuer die Operanden. Der oberste Wert im Operandenstack repraesentiert
	den Akku der InputDevice. */
	private Stack ops;
	
	private Op op;								/** Durchzufuehrende Operation */
	
	/** Beinhaltet zu den Zustaenden die jeweilige textuelle Beschreibung*/
	private final static String [] stateString = { "Initialize the inputDevice by entering your agentID:" ,
	"ready for orders, press button 'order'", "(origin) table.customer: enter table", 
	"OriginENDE never visible: Gast", "(origin) table.customer: enter customer",
	"(orderitems) articleID, quantity: enter artikelID", "(orderitems) articleID, quantity: enter quantity",
	"transfer successfully done"};

	
	//// Constructors

	/**	Post:	Erzeugt und Initialisiert ein [InputDeviceEngine]-Objekt. <BR>
		Prec:	-
	@param	-
	@see	-
	@init	NS, 12-April-2000
	@curr	NS, 12-April-2000 */
	InputDeviceEngine()
		{ 
		this.clear(); 
		state = 0;
		this.orderVector = new Vector();
		notifyObservers();
		}
	

	//// Zustands-Zugriffs-Methoden
	
	/**	Post:	Antwortet mit dem gesamten Operandenstack. Op.top() repraesentiert
				den Akku der InputDevice. <BR>
		Prec:	-
	@param
	@see	java.util.Stack
	@init	NS, 12-April-2000
	@curr	NS, 12-April-2000 */
	Stack ops()
		{ return this.ops; }
	

	/**	Post:	Setzt den Operandenstack auf [aStack]. Bitte beachten, die Methode
				setzt den _gesamten_ Stack und nicht nur das oberste Element. <BR>
		Prec:	-

	@param	aStack 	Neu zu setzender Operandenstack.
	@see	-
	@init	NS, 12-April-2000
	@curr	NS, 12-April-2000 */
	void ops( Stack aStack )
		{ this.ops = aStack; }


	/**	Post:	Antwortet mit dem aktuell gesetzten Operator. <BR>
		Prec:	-

	@param	-
	@see	-
	@init	NS, 12-April-2000
	@curr	NS, 12-April-2000 */
	Op op()
		{ return this.op; }
		
	/**	Post:	Antwortet mit dem zum jeweiligen Status 
					entsprechenden String in 'stateString'.<BR>
		Prec:	-
	@param	-
	@see	-
	@init	NS, 12-April-2000
	@curr	NS, 12-April-2000 */
	private String state()
		{ return stateString[ this.state ]; }
		
		
	/**	Post:	Setzt den Operator. <BR>
		Prec:	-
	@param	anOp 	Neu zu setzender Operator.
	@see	-
	@init	NS, 12-April-2000
	@curr	NS, 12-April-2000 */
	void op( Op anOp )
		{ this.op = anOp; }
		
	/**	Post:	gibt das aktuelle Order-Objekt zurueck.<BR>
		Prec:	-
	@param	-
	@see	-
	@init	NS, 12-April-2000
	@curr	NS, 12-April-2000 */
	private Order order()
		{ return this.order; }
		
	/**	Post:	Setzt das aktuelle Order-Objekt.<BR>
		Prec:	-
	@param	-
	@see	-
	@init	NS, 12-April-2000
	@curr	NS, 12-April-2000 */
	private void order( Order anOrder )
		{ this.order = anOrder; }


	/**	Post:	Gibt den orderVector zurueck.<BR>
		Prec:	-
	@param	-
	@see	-
	@init	NS, 12-April-2000
	@curr	NS, 12-April-2000 */
	private Vector orderVector()
		{ return this.orderVector; }
		
	/**	Post:	Gibt den orderItemsVector zurueck.<BR>
		Prec:	-
	@param	-
	@see	-
	@init	NS, 12-April-2000
	@curr	NS, 12-April-2000 */
	private Vector orderItemsVector()
		{ return this.orderItemsVector; }
		

	/**	Post:	Gibt die ID des Agenten zureuck.<BR>
		Prec:	-
	@param	-
	@see	-
	@init	NS, 12-April-2000
	@curr	NS, 12-April-2000 */
	private int agent()
		{ return this.agent; }
	

	/**	Post:Setzt den Agenten.<BR>
		Prec:	-
	@param	-
	@see	-
	@init	NS, 12-April-2000
	@curr	NS, 12-April-2000 */
	private void agent( int anAgentID )
		{ this.agent = anAgentID; }
		

	//// Instance-Methods
	
	/**	Post:	Antwortet mit True, falls 0 Wert eingegeben wurde.<BR>
		Prec:	-
	@param	-
	@see	-
	@init	NS, 12-April-2000
	@curr	NS, 12-April-2000 */
	private boolean isZeroValue()
		{
		
		if ( ops().empty() )
			{ return false;}
			
		String temp;
		
		// Aktuellen Wert des Akkus vom Stack holen und in String umwandeln
		temp = (String) ops().peek();
	
		if ( ( ( state==0 ) || ( state==2 ) || ( state==4 ) || ( state==5 ) || ( state==6 ) ) &&
			(! ( ( temp.equalsIgnoreCase("00") || temp.equalsIgnoreCase("0") ) ) ) )
			{ 
			return false; // Keine 0-Werte alleine zulassenwieder zurueckschreiben
			}
		return true;
		}
		
	
	/**	Post:	Erstellt ein neues Order-Objekt.<BR>
		Prec:	-
	@param	-
	@see	-
	@init	NS, 12-April-2000
	@curr	NS, 12-April-2000 */
	private void createElement()
		{ 
		try
			{
			this.order( ( Order ) Order.class.newInstance() ); 
			this.orderItemsVector = new Vector();
			}
		catch ( IllegalAccessException e )
			{ System.out.println( e ) ;}
		catch ( InstantiationException e )
			{ System.out.println( e );}
		}
		
	/**	Post:	Fuegt dem orderVector ein neues Order-Objekt hinzu.<BR>
		Prec:	-
	@param	-
	@see	-
	@init	NS, 12-April-2000
	@curr	NS, 12-April-2000 */
	private void addToOrders( Order anOrder )
		{ this.orderVector().addElement( anOrder ); }
		
	/**	Post:	Fuegt dem orderItemsVector einen neuen OrderItemPart 
					[Artikel, Quantitaet] hinzu.<BR>
		Prec:	-
	@param	anOrderItemPart  Artikel oder dessen Menge (Quantitaet)
	@see	-
	@init	NS, 12-April-2000
	@curr	NS, 12-April-2000 */
	private void addToOrderItems( Object anOrderItemPart )
		{ this.orderItemsVector().addElement( anOrderItemPart ); }		
		
		
	/**	Post:	Schliesst den aktuellen Order ab, indem dem orderItemsVector 
					der neuen OrderItemPart [Quantitaet] hinzugefuegt und der
					orderVector gefuellt wird.
					<BR>
		Prec:	-
	@param -
	@see	-
	@init	NS, 16-April-2000
	@curr	NS, 16-April-2000 */		
	private void finishOrder()
		{
		this.addToOrderItems( ops.pop().toString() ); 				//Elemente in orderItems setzen
		this.order().setOrderItems( orderItemsVectorToArray() ); // OrderItems in Order speichern
		this.addToOrders( this.order() ); 								//aktuellen Order in Vector speichern
		}

	/**	Post:	Initialisiert ein neues Order-Objekt und setzt via dessen
					setter-Methoden die essentiellen Anfangs-Werte.<BR>
		Prec:	-
	@param -
	@see	-
	@init	NS, 16-April-2000
	@curr	NS, 16-April-2000 */		
	private void initializeOrder()
		{
		this.createElement();
		this.order().setAgent( this.agent() );	  // setzen des Agents
		this.order().setOrderTime( new Date() ); // setzen der Systemzeit
		}

	/**	Post:	Fuegt alle OrderItems der Reihe nach aus dem Vector in
					ein neues Array und antwortet mit diesem Array. Die aus
					dem Vector ausgelesenen Elemente muessen zuerst in den
					Typ des Arrays [int] konvertiert werden..<BR>
		Prec:	-
	@param	-
	@see	-
	@init	NS, 12-April-2000
	@curr	NS, 12-April-2000 */
	public int[] orderItemsVectorToArray()
		{
		if ( this.orderItemsVector().size() % 2 == 1 )
			{ 
			System.out.println( "ERROR in Anzahl der OrderItems: Alles wird zurueck gesetzt" );
			this.clear();
			}
		int[] tmpOrderItems = new int [ this.orderItemsVector().size() ];
		for( int i = 0; i < this.orderItemsVector().size(); i++ )
			{ 
			tmpOrderItems[i] = convertStringToInt( this.orderItemsVector().elementAt(i).toString() );
			}
		this.orderItemsVector().removeAllElements();
		return tmpOrderItems;
		}

	/**	Post:	Benachrichtigt alle Observer, dass sich der Wert des Akkus
					geaendert hat und liefert den neuen Wert als Argument fuer
					die notifyObserver-Nachricht mit. <BR>
		Prec:	-
	@param	-
	@see	-
	@init	NS, 12-April-2000
	@curr	NS, 12-April-2000 */
	public void notifyObservers()
		{
		setChanged();
		notifyObservers(  new Op( ops().peek().toString(), this.state() ) );
		}
		

	/**	Post:	Teilt der Engine mit, dass die Dezimalpunktaste gedrueckt wurde und
				fuehrt die notwendigen Aktionen und Zustandwechsel durch. <BR>
		Prec:	-
	@param	-
	@see	-
	@init	NS, 12-April-2000
	@curr	NS, 12-April-2000 */
	synchronized void period()
		{
		String temp;
		if ( ( !( isZeroValue()) ) && ( state == 2 ) ) 
			{	// Wenn Zustand "Origin, Eingabe Vorkommastellen von Origin ", dann
				// neuer Zustand "Origin, Eingabe Nachkommastellen (von Origin) gewuenscht (Point)"
			temp = (String) ops().pop();
			temp = temp + ".";
			// Wichtig: geht hier direkt auf State 4 ueber, da State 3 akzeptiert
			state = 4;
			ops.push( temp );		// neuer Akku-Wert wieder auf Stack
			notifyObservers();	// Es hat sich was veraendert -> Observer benachrichtigen
			}
		}
		

	/**	Post:	Teilt der Engine mit, dass eine Zifferntaste gedrueckt wurde. 
				Fuegt eine Ziffer zum aktuell im Akku befindlichen Wert hinzu und
				fuehrt die ggf. notwendigen Zustandswechsel durch. <BR>
		Prec:	-

	@param	-
	@see	-
	@init	NS, 12-April-2000
	@curr	NS, 12-April-2000 */
	synchronized void digit( String aString )
		{
		String temp;
		// Wenn zuvor "Transfer" gedrueckt wurde, dann vor Eingabe einer
		// neuen Ziffer den Rechner initialisieren
		if ( state == 7 ) 
			{ clear(); };
			
		// Aktuellen Wert des Akkus vom Stack holen und in String umwandeln
		temp = (String) ops().pop();
		
		if ( state == 1 )
			{ // Gelangt hier hin nach 'Cancel', tue nichts in diesem Status
			ops().push( "0" );
			}
		else
			{	// Ansonsten Ziffer an den alten Wert des Akkus anhaengen und Zust. beibehalten
			temp = temp + aString;
			// Abbruch, wenn Status nicht erlaubt (0-Werte)
			if ( isZeroValue() )
				{ return; } 
			// ggf. fuehrende Null eliminieren
			if ( temp.startsWith( "0" ) )	// zu implementieren
				{ temp = temp.substring(1); }
			};

		ops.push( temp );		// neuer Akku-Wert wieder auf Stack
		notifyObservers(); 	// Es hat sich was veraendert -> Observer benachrichtigen
		}
		
/**	Post:	Enter-Taste wurde gedrueckt. Berechnung durchfuehren und ggf. Zustandswechsel 
				vornehmen.<BR>
		Prec:	-
	@param	-
	@see	-
	@init	NS, 12-April-2000
	@curr	NS, 12-April-2000 */
	synchronized void enter()
		{
		if ( isZeroValue() )
			{ return; }
		if ( state == 0 )
			{ // einmalige initialisierung des Agents
			this.agent( convertStringToInt(ops.peek().toString()) );
			ops().push( "0" );
			state = 1;
			}
		else if ( state == 4 ) 
			{ // Setze Origin des aktuellen Orders
			this.order().setOrigin( convertStringToFloat(ops.peek().toString()) );
			ops().push( "0" );
			state = 5;
			}
		else if ( state == 5 ) 
			{ // lege in orderItems ab (ArticleID)
			this.addToOrderItems( ops.peek() );
			ops().push( "0" );
			state = 6;
			}
		else if ( state == 6 ) 
			{ // lege in orderItems ab (Quantity)
			this.addToOrderItems( ops.peek() );
			ops().push( "0" );
			// Gehe zurueck auf State 5
			state = 5;
			}
		notifyObservers();
		}
				
	
	/**	Post:	Reset der InputDeviceEngine. <BR>
		Prec:	-
	@param	-
	@see	-
	@init	NS, 12-April-2000
	@curr	NS, 12-April-2000, nur wenn state > 0 auf state=1 wechseln.*/
	synchronized void clear()
		{
		// Stack loeschen
		ops( new Stack() );
		// Akku initialisieren
		ops().push( "0" );
		// Operationsspeicher loeschen
		op( null );
		// Bereit
		if ( state > 0 )
			{ state = 1; }
		notifyObservers();
		}

	
	/**	Post:	Transfertaste wurde gedrueckt. Abschliessende Berechnung durchfuehren
					durch Status-Sicherung des aktuellen Order-Objektes und schliesslich Transfer 
					an DockingStation, indem alle gespeicherten Orders im ordersVector an Dockingstation 
					uebergeben werden. OrderVector() neu initialisieren und State (7) setzen.<BR>
		Prec:	-
	@param	-
	@see	-
	@init	NS, 12-April-2000
	@curr	NS, 12-April-2000, Transfer implementiert.*/
	synchronized void transfer() {};
	
	/*
		{
		if ( state == 6 )
			{
			// speichere den letzten Eintrag resp. den aktuellen Order in orderVector()
			this.finishOrder();
			// Transfer wird unternommen, indem alle Elemente ausgelesen und schliesslich 
			// geloescht werden.
			if (! orderVector.isEmpty() )
				{
				for ( int i = 0; i <  this.orderVector().size(); i++ )
					{ sopra.DockingStation.putOrder( ( Order )this.orderVector().elementAt(i) ); }
				this.orderVector().removeAllElements();
				}
			this.clear();
			state = 7;
			notifyObservers();
			}
		}
	*/
	/**	Post:	enterOrder-Taste wurde gedrueckt. Falls der Zustand dies aktzeptiert, wird
					Engine wieder initialisiert indem ein neues Order-Objekt erstellt wird.<BR>
		Prec:	-
	@param	-
	@see	-
	@init	NS, 12-April-2000
	@curr	NS, 12-April-2000 */
	synchronized void enterOrder()
		{
		// nur diese 3 Zustaende werden hier akzeptiert
		if ( ( state == 1 ) || ( ( state == 6 ) && (! isZeroValue()) ) || ( state == 7 ) )
			{
			if ( state == 6 ) 	
				{ this.finishOrder(); }
			this.initializeOrder();
			this.clear();
			state = 2;
			notifyObservers();
			}
		}	

/**	Post:	Hilfsmethode, um String-Objekte in primitiven Datentyp 'int' zu verwandeln.<BR>
		Prec:	-
	@param	valueAsString String
	@see		inspection.StringToObjectConverter.Convert()
	@init	NS, 17-Maerz-2000
	@curr	NS, 17-Maerz-2000 */
	private int convertStringToInt( String valueAsString )
	
		{
		try
			{
			Integer idObject = (Integer) util.StringToObjectConverter.Convert( valueAsString, int.class );
			return idObject.intValue();
			}
		catch( NumberFormatException e )
			{ System.out.println( e ); }
		return -1;
		}


	/**	Post: Hilfsmethode, um String-Objekte in primitiven Datentyp 'float' zu verwandeln.	<BR>
		Prec:	-
	@param	valueAsString String, der zu einem float konvertiert werden soll
	@see	inspection.StringToObjectConverter.Convert()
	@init	NS, 17-Maerz-2000
	@curr	NS, 17-Maerz-2000 */
	private float convertStringToFloat( String valueAsString )
		{
		try
			{
			Float idObject = (Float) util.StringToObjectConverter.Convert( valueAsString, float.class );
			return idObject.floatValue();
			}
		catch( NumberFormatException e )
			{ System.out.println( e ); }
		return -1;
		}	
	}