package sopra;

import java.util.GregorianCalendar;
import java.util.Calendar;
import java.util.Random;

/**
 * Class is responsible for the price calculation, implementing also
 * a price sensitive correction factor
 * 
 * Variant "Taugwalder"
 * 
 * @author Reto Geiger
 */
public class CafeTest {

	private float pe;
	//Gesamtumsatz pro PriceCalculator
	private float[] totalRevenue = { 0f, 0f, 0f, 0f, 0f, 0f, 0f, 0f };
	//Alle gehandelten Artikel
	private Article[] allArticles = new Article[10];
	private GregorianCalendar time;

	final static int maximalChange = 100;

	//Nchste bestellzeit berechnen	
	Random randomTime = new Random();

	//Artikel
	/*
		private Article beer;
		private Article wine;
		private Article redwine;
		private Article bratwurst;
		private Article cordonbleu;
		private Article salad;
		private Article schnaps;
		private Article cola;
		private Article icetea;
		private Article redbull;
	*/

	//PriceCalculators
	private PriceCalculator tempCalc;
	private PriceCalculator1 calc1;
	private PriceCalculator2 calc2;
	private PriceCalculator3 calc3;
	private PriceCalculator4 calc4;
	private PriceCalculator5 calc5;
	private PriceCalculator6 calc6;
	private PriceCalculator7 calc7;
	private PriceCalculator8 calc8;

	public CafeTest() {
		//Preiselastizitt
		pe = 3.5f;

		//Calculators initialisieren
		tempCalc = new PriceCalculator();
		calc1 = new PriceCalculator1();
		calc2 = new PriceCalculator2();
		calc3 = new PriceCalculator3();
		calc4 = new PriceCalculator4();
		calc5 = new PriceCalculator5();
		calc6 = new PriceCalculator6();
		calc7 = new PriceCalculator7();
		calc8 = new PriceCalculator8();

		//Setze erste Bestellung auf 11:00 Uhr
		time = new GregorianCalendar(2004, 6, 22, 11, 00);

		//10 Artikel festlegen
		//Cardinal Draft erste Bestellung um 11:05 Uhr
		// durch drei teilen, weil Menge von 11-14h
		//beer = new Article(4.5f, 50f / 3, 3.5f, 5.5f, new GregorianCalendar(2004, 6, 22, 11, 5));
		allArticles[0] = new Article(4.5f, 50f / 3, 3.5f, 5.5f, new GregorianCalendar(2004, 6, 22, 11, 5));

		//Weisswein erste Bestellung um 11:23 Uhr
		//durch drei teilen, weil Menge von 11-14h
		//durch zwei teilen, weil zwei verschiedene Weine
		//wine = new Article(38f, 10f / (3 * 2), 30f, 46f, new GregorianCalendar(2004, 6, 22, 11, 23));
		allArticles[1] = new Article(38f, 10f / (3 * 2), 35f, 46f, new GregorianCalendar(2004, 6, 22, 11, 23));

		//Rotwein erste Bestellung um 11:16 Uhr
		//durch drei teilen, weil Menge von 11-14h
		//durch zwei teilen, weil zwei verschiedene Weine
		//redwine = new Article(60f, 10f / (3 * 2), 45f, 75f, new GregorianCalendar(2004, 6, 22, 11, 16));
		allArticles[2] = new Article(60f, 10f / (3 * 2), 45f, 75f, new GregorianCalendar(2004, 6, 22, 11, 16));

		//Bratwurst erste Bestellung um 11:15 Uhr
		//durch drei teilen, weil Menge von 11-14
		//durch drei teilen, weil drei verschiedene Speisen
		//bratwurst = new Article(19.5f, 90f / (3 * 3), 15f, 25f, new GregorianCalendar(2004, 6, 22, 11, 15));
		allArticles[3] = new Article(19.5f, 90f / (3 * 3), 15f, 25f, new GregorianCalendar(2004, 6, 22, 11, 15));

		//Cordon Bleu erste Bestellung um 11:07 Uhr
		//durch drei teilen, weil Menge von 11-14
		//durch drei teilen, weil drei verschiedene Speisen
		//cordonbleu = new Article(29.5f, 90f / (3 * 3), 22.5f, 36.5f, new GregorianCalendar(2004, 6, 22, 11, 7));
		allArticles[4] = new Article(29.5f, 90f / (3 * 3), 25.5f, 36.5f, new GregorianCalendar(2004, 6, 22, 11, 7));

		//gemischter Salat erste Bestellung um 11:03 Uhr
		//durch drei teilen, weil Menge von 11-14
		//durch drei teilen, weil drei verschiedene Speisen
		//salad = new Article(8.5f, 90f / (3 * 3), 6.5f, 10.5f, new GregorianCalendar(2004, 6, 22, 11, 3));
		allArticles[5] = new Article(8.5f, 90f / (3 * 3), 5f, 10.5f, new GregorianCalendar(2004, 6, 22, 11, 3));

		//Pastis erste Bestellung um 11:10 Uhr
		//durch drei teilen, weil Menge von 11-14
		//schnaps = new Article(6.0f, 15f / 3, 4.5f, 7.5f, new GregorianCalendar(2004, 6, 22, 11, 10));
		allArticles[6] = new Article(6.0f, 15f / 3, 5.5f, 7.5f, new GregorianCalendar(2004, 6, 22, 11, 10));

		//Cola erste Bestellung um 11:01 Uhr
		//durch drei teilen, weil Menge von 11-14
		//durch drei teilen, weil drei verschiedene alkoholfreie Getrnke
		//cola = new Article(3.5f, 200f / (3 * 3), 2.5f, 4.5f, new GregorianCalendar(2004, 6, 22, 11, 1));
		allArticles[7] = new Article(3.5f, 200f / (3 * 3), 2.5f, 4.5f, new GregorianCalendar(2004, 6, 22, 11, 1));

		//Cola erste Bestellung um 11:05 Uhr
		//durch drei teilen, weil Menge von 11-14
		//durch drei teilen, weil drei verschiedene alkoholfreie Getrnke
		//icetea = new Article(3f, 200f / (3 * 3), 2.1f, 3.9f, new GregorianCalendar(2004, 6, 22, 11, 5));
		allArticles[8] = new Article(3f, 200f / (3 * 3), 2.1f, 3.9f, new GregorianCalendar(2004, 6, 22, 11, 5));

		//Cola erste Bestellung um 11:08 Uhr
		//durch drei teilen, weil Menge von 11-14
		//durch drei teilen, weil drei verschiedene alkoholfreie Getrnke
		//redbull = new Article(4f, 200f / (3 * 3), 3.1f, 4.9f, new GregorianCalendar(2004, 6, 22, 11, 8));
		allArticles[9] = new Article(4f, 200f / (3 * 3), 3.1f, 4.9f, new GregorianCalendar(2004, 6, 22, 11, 8));
	}

	/*
	 * Ablauf:
	 * 1. 	Sinnvoller Zufallswert fr Menge als Ausgangswert
	 * 2. 	Iteration ber 8 Preisberechnungsklassen
	 * 			- calculatePrice aufrufen
	 * 			- mit neuem Preis neue Menge berechnen
	 *
	public static void main(String[] args) {
		//TEST
		//System.out.println("Hallo");
		CafeTest cafeTest = new CafeTest();
		for (int i = 0; i < 600; i++) {
			float[] revs = cafeTest.calculate();
			//for(int j=0;j<8;j++) {
			//	System.out.println("Revenue["+j+"]: "+revs[j]);
			//} 
		}
		float[] revs = cafeTest.getTotalRevenue();
		for (int j = 0; j < 8; j++) {
			System.out.println("Revenue[" + j + "]: " + revs[j]);
		}
		//ENDE TEST
	}*/

	/*
	 * Berechnet neue Verkaufsmenge innerhalb des gleichen intervalls wie die letzte Menge.
	 * 
	 * @param pe - Preiselastizitt
	 * @param standardPrice - Listenpreis
	 * @param currentPrice - momentaner Preis
	 * @param amountSold - Verkaufsmenge hochgerechnet auf letzte Stunde
	 */
	public float calculateNewAmount(float standardPrice, float currentPrice, float amountSold) {
		//Verkaufsmenge im nchsten Zeitintervall hochgerechnet auf eine Stunde
		//float newAmount;
		//"falsche" Berechnung mit richtigem Resultat
		float newAmount = amountSold - (amountSold * this.pe * (currentPrice - standardPrice)) / currentPrice;
		//"richtige" Berechnung mit falschem Resultat
		//float newAmount = amountSold - (amountSold * this.pe * (standardPrice - currentPrice)) / standardPrice;

		//return newAmount;
		// newAmount zurckliefern, nderung darf aber nicht grsser als +/- CafeTest.maximalChange sein
		if (newAmount - amountSold > CafeTest.maximalChange)
			return amountSold + CafeTest.maximalChange;
		else if (amountSold - newAmount > CafeTest.maximalChange)
			return amountSold - CafeTest.maximalChange;
		else
			return newAmount;
	}

	//Interface fr GUI
	public float[] calculate() {

		//Zufllige normalverteilte Schwankung berechnen (Mittelwert 1, Standardabweichung 1)
		//und auf int runden (ergibt effektiven Mittelwert von ~0
		double randomAmount = randomTime.nextGaussian() + 1;

		//ber alle Artikel iterieren
		for (int j = 0; j < 10; j++) {

			Article currentArticle = allArticles[j];

			//Durchschnittliche Verkaufsmengen an Tageszeit anpassen 14:00 Uhr
			if ((time.get(Calendar.HOUR_OF_DAY) == 14) && (time.get(Calendar.MINUTE)) == 0) {

				switch (j) {
					//Bier ndern 
					//durch drei teilen, weil Intervall 3 Stunden hat
					case (0) :
						allArticles[j].setStandardAmountSold(200f / 3);
						break;
						//Weisswein ndern 
						//durch drei teilen, weil Intervall 3 Stunden hat
						//durch zwei teilen, weil zwei Weine verkauft werden
					case (1) :
						allArticles[j].setStandardAmountSold(10f / (3 * 2));
						break;
						//Rotwein ndern 
						//durch drei teilen, weil Intervall 3 Stunden hat
						//durch zwei teilen, weil zwei Weine verkauft werden
					case (2) :
						allArticles[j].setStandardAmountSold(10f / (3 * 2));
						break;
						//Bratwurst ndern 
						//durch drei teilen, weil Intervall 3 Stunden hat
						//durch drei teilen, weil drei Speisen verkauft werden
					case (3) :
						allArticles[j].setStandardAmountSold(10f / (3 * 3));
						break;
						//Cordon Bleu ndern 
						//durch drei teilen, weil Intervall 3 Stunden hat
						//durch drei teilen, weil drei Speisen verkauft werden
					case (4) :
						allArticles[j].setStandardAmountSold(10f / (3 * 3));
						break;
						//Salat ndern 
						//durch drei teilen, weil Intervall 3 Stunden hat
						//durch drei teilen, weil drei Speisen verkauft werden
					case (5) :
						allArticles[j].setStandardAmountSold(10f / (3 * 3));
						break;
						//Pastis ndern 
						//durch drei teilen, weil Intervall 3 Stunden hat
					case (6) :
						allArticles[j].setStandardAmountSold(10f / (3));
						break;
						//Cola ndern 
						//durch drei teilen, weil Intervall 3 Stunden hat
						//durch drei teilen, weil drei Speisen verkauft werden
					case (7) :
						allArticles[j].setStandardAmountSold(200f / (3 * 3));
						break;
						//Ice Tea ndern 
						//durch drei teilen, weil Intervall 3 Stunden hat
						//durch drei teilen, weil drei Speisen verkauft werden
					case (8) :
						allArticles[j].setStandardAmountSold(200f / (3 * 3));
						break;
						//Red Bull ndern 
						//durch drei teilen, weil Intervall 3 Stunden hat
						//durch drei teilen, weil drei Speisen verkauft werden
					case (9) :
						allArticles[j].setStandardAmountSold(200f / (3 * 3));
						break;
				}
			}

			//Durchschnittliche Verkaufsmengen an Tageszeit anpassen 17:00 Uhr
			else if ((time.get(Calendar.HOUR_OF_DAY) == 17) && (time.get(Calendar.MINUTE)) == 0) {

				switch (j) {
					//Bier ndern 
					//durch vier teilen, weil Intervall 4 Stunden hat
					case (0) :
						allArticles[j].setStandardAmountSold(800f / 4);
						break;
						//Weisswein ndern 
						//durch vier teilen, weil Intervall 4 Stunden hat
						//durch zwei teilen, weil zwei Weine verkauft werden
					case (1) :
						allArticles[j].setStandardAmountSold(40f / (4 * 2));
						break;
						//Rotwein ndern 
						//durch vier teilen, weil Intervall 4 Stunden hat
						//durch zwei teilen, weil zwei Weine verkauft werden
					case (2) :
						allArticles[j].setStandardAmountSold(40f / (4 * 2));
						break;
						//Bratwurst ndern 
						//durch vier teilen, weil Intervall 4 Stunden hat
						//durch drei teilen, weil drei Speisen verkauft werden
					case (3) :
						allArticles[j].setStandardAmountSold(50f / (4 * 3));
						break;
						//Cordon Bleu ndern 
						//durch vier teilen, weil Intervall 4 Stunden hat
						//durch drei teilen, weil drei Speisen verkauft werden
					case (4) :
						allArticles[j].setStandardAmountSold(50f / (4 * 3));
						break;
						//Salat ndern 
						//durch vier teilen, weil Intervall 4 Stunden hat
						//durch drei teilen, weil drei Speisen verkauft werden
					case (5) :
						allArticles[j].setStandardAmountSold(50f / (4 * 3));
						break;
						//Pastis ndern 
						//durch vier teilen, weil Intervall 4 Stunden hat
					case (6) :
						allArticles[j].setStandardAmountSold(90f / (4));
						break;
						//Cola ndern 
						//durch vier teilen, weil Intervall 4 Stunden hat
						//durch drei teilen, weil drei Speisen verkauft werden
					case (7) :
						allArticles[j].setStandardAmountSold(200f / (4 * 3));
						break;
						//Ice Tea ndern 
						//durch vier teilen, weil Intervall 4 Stunden hat
						//durch drei teilen, weil drei Speisen verkauft werden
					case (8) :
						allArticles[j].setStandardAmountSold(200f / (4 * 3));
						break;
						//Red Bull ndern 
						//durch vier teilen, weil Intervall 4 Stunden hat
						//durch drei teilen, weil drei Speisen verkauft werden
					case (9) :
						allArticles[j].setStandardAmountSold(200f / (4 * 3));
						break;
				}
			}

			//Berechne Bestellungen, die in einer Minute Realzeit = 1 Sekund Sim anfallen
			if (currentArticle.getOrderTime().equals(time)) {
				//Bier wurde in dieser Minute bestellt, Bestellung verrechnen
				float[] currentPrices = currentArticle.getCurrentPrices();
				//Bier wurde in dieser Minute bestellt, Bestellung verrechnen
				float[] standardPrices = currentArticle.getStandardPrices();
				//alte Preise
				float[] oldPrices = currentArticle.getOldPrices();
				//Verkaufsmengen hochgerechnet auf eine Stunde
				float[] amountsSold = currentArticle.getAmountsSold();
				Calendar lastOrder = currentArticle.getLastOrderTime();

				for (int i = 0; i < 8; i++) {

					//Zeitdifferenz zwischen aktueller und vorheriger Bestellung berechnen
					int diffMinutes;

					if (lastOrder.get(Calendar.HOUR_OF_DAY) < time.get(Calendar.HOUR_OF_DAY)) {
						//Letzte bestellung wurde in vorangehender Stunde aufgegeben.
						//Wieviele Stunden liegen dazwischen?
						int diffHour = time.get(Calendar.HOUR_OF_DAY) - lastOrder.get(Calendar.HOUR_OF_DAY);
						int currentMinutes = time.get(Calendar.MINUTE) + 60 * diffHour;

						//Differenz in Minuten
						diffMinutes = currentMinutes - lastOrder.get(Calendar.MINUTE);
						//System.out.println("time.MINUTE: "+time.get(Calendar.MINUTE));
					} else {
						//Bestellungen in gleicher Stunde:
						//ACHTUNG: keine Behandlung von lastOrder spter als time!
						diffMinutes = time.get(Calendar.MINUTE) - lastOrder.get(Calendar.MINUTE);
					}

					//mit Verkaufter Menge (letzte Periode) und Preis (etzte Periode)
					//Umsatz berechnen
					//float newRevenue = 0f;
					//float newRevenue = amountsSold[i] * diffMinutes / 60 * currentPrices[i];

					//mit Verkaufter Menge (letzte Periode) und Preis (etzte Periode)
					//Umsatz berechnen
					float newRevenue = 0f;
					//newRevenue = amountsSold[i]*diffMinutes/60*currentPrices[i];
					//Gewinn ((Umsatz-Kosten)*Zeitintervall*Menge) Berechnen
					newRevenue = amountsSold[i] * diffMinutes / 60 * (currentPrices[i] - currentArticle.getCost());

					//Umsatz speichern
					totalRevenue[i] = totalRevenue[i] + newRevenue;

					//Neue VerkaufsMenge berechnen
					//float newAmountPerHour = calculateNewAmount(standardPrices[i], currentPrices[i], amountsSold[i]);

					//**********************************************************
					//Neue VerkaufsMenge berechnen
					//float newAmountPerHour = calculateNewAmount(currentPrices[i],
					//											oldPrices[i],amountsSold[i]);
					float newAmountPerHour =
						calculateNewAmount(
							currentArticle.getStandardPrice(),
							currentPrices[i],
							currentArticle.getStandardAmountSold());

					//Neue Verkaufsmenge/Stunde im Artikel Speichern
					currentArticle.setAmount(newAmountPerHour, i);

					//zufllige Schwankung hinzufgen
					float orderAmount = (newAmountPerHour + (float) randomAmount) * diffMinutes / 60;

					//Neue Menge auf int runden und Intervall bercksichtigen
					int intAmount = new Float(orderAmount).intValue();

					//berprfen, ob Amount > 0; wenn <0, dann auf 0 setzen -> keine Bestellung in diesem Fall
					if (intAmount < 0) {
						intAmount = 0;
					}
					//************************************************

					//Neue Verkaufsmenge/Stunde im Artikel Speichern
					currentArticle.setAmount(newAmountPerHour, i);

					//Neue Menge auf int runden und Intervall bercksichtigen
					//int intAmount = new Float(newAmountPerHour).intValue() * diffMinutes / 60;

					//Neuer Preis berechnen 
					float newPrice = 0f;
					switch (i) {
						case (0) :
							newPrice =
								calc1.calculatePrice(
									currentArticle.getStandardPrice(),
									currentPrices[i],
									currentArticle.getLowerBoundary(),
									currentArticle.getUpperBoundary(),
									intAmount,
									currentArticle.getStandardAmountSold(),
									time);
							break;
						case (1) :
							newPrice = currentArticle.getLowerBoundary();
							/**	calc2.calculatePrice(
									currentArticle.getStandardPrice(),
									currentPrices[i],
									currentArticle.getLowerBoundary(),
									currentArticle.getUpperBoundary(),
									intAmount,
									currentArticle.getStandardAmountSold(),
									time);*/
							break;
						case (2) :
							newPrice =
								calc3.calculatePrice(
									currentArticle.getStandardPrice(),
									currentPrices[i],
									currentArticle.getLowerBoundary(),
									currentArticle.getUpperBoundary(),
									intAmount,
									currentArticle.getStandardAmountSold(),
									time);
							break;
						case (3) :
							newPrice =
								calc4.calculatePrice(
									currentArticle.getStandardPrice(),
									currentPrices[i],
									currentArticle.getLowerBoundary(),
									currentArticle.getUpperBoundary(),
									intAmount,
									currentArticle.getStandardAmountSold(),
									time);
							break;
						case (4) :
							newPrice =
								calc5.calculatePrice(
									currentArticle.getStandardPrice(),
									currentPrices[i],
									currentArticle.getLowerBoundary(),
									currentArticle.getUpperBoundary(),
									intAmount,
									currentArticle.getStandardAmountSold(),
									time);
							break;
						case (5) :
							newPrice =
								calc6.calculatePrice(
									currentArticle.getStandardPrice(),
									currentPrices[i],
									currentArticle.getLowerBoundary(),
									currentArticle.getUpperBoundary(),
									intAmount,
									currentArticle.getStandardAmountSold(),
									time);
							break;
						case (6) :
							newPrice =
								calc7.calculatePrice(
									currentArticle.getStandardPrice(),
									currentPrices[i],
									currentArticle.getLowerBoundary(),
									currentArticle.getUpperBoundary(),
									intAmount,
									currentArticle.getStandardAmountSold(),
									time);
							break;
						case (7) :
							newPrice =
								calc8.calculatePrice(
									currentArticle.getStandardPrice(),
									currentPrices[i],
									currentArticle.getLowerBoundary(),
									currentArticle.getUpperBoundary(),
									intAmount,
									currentArticle.getStandardAmountSold(),
									time);
							break;
						default :
							newPrice =
								tempCalc.calculatePrice(
									currentArticle.getStandardPrice(),
									currentPrices[i],
									currentArticle.getLowerBoundary(),
									currentArticle.getUpperBoundary(),
									intAmount,
									currentArticle.getStandardAmountSold(),
									time);
					}
					//currentArticle.setOldPrice(currentPrices[i], i);
					//Neuer Preis im Artikel speichern
					currentArticle.setPrice(newPrice, i);
				}

				//Nchste Bestellung zwischen 5 und 25 Minuten nach dieser Bestellung
				int minuteAdd = randomTime.nextInt(26) + 5;
				GregorianCalendar tempTime =
					new GregorianCalendar(
						time.get(Calendar.YEAR),
						time.get(Calendar.MONTH),
						time.get(Calendar.DAY_OF_MONTH),
						time.get(Calendar.HOUR_OF_DAY),
						time.get(Calendar.MINUTE));
				tempTime.add(Calendar.MINUTE, minuteAdd);

				//Nchste Bestellung setzen
				currentArticle.setNextOrder(tempTime);

				//Letzte Bestellung auf aktuelle Zeit Schalten
				currentArticle.setLastOrder(
					new GregorianCalendar(
						time.get(Calendar.YEAR),
						time.get(Calendar.MONTH),
						time.get(Calendar.DAY_OF_MONTH),
						time.get(Calendar.HOUR_OF_DAY),
						time.get(Calendar.MINUTE)));
			}
		}

		//Zeit eine Minute weiterschalten
		time.add(Calendar.SECOND, 6);

		//gib Array mit allen Einknften zurck
		return getTotalRevenue();
	}

	private float[] getTotalRevenue() {
		return totalRevenue;
	}
}
