package sopra;
import java.util.Calendar;

/**
 * 	RIRO-SOFT
 */

/**
* This class contains methods and attributes needed to
* calculate the current price of variable priced articles.
* The price is calculated on each order.
*
* @author 				Eberli
* @copyright 			Department for Computer Science, Requirement
* 						Engineering Group
* @history 				2004-24-03, design
* 						2004-20-05, design
* 						2004-23-05, implementation 1.0
* 						2004-04-06, implementation 1.1: lowerBoundary <= currentPrice <= upperBoundary
* 						2004-18-06, implementation 1.2: totalPriceFall * 0.2
* @version 				2004-18-06, 1.2
* @responsibilities 	This class calculates prices for variable
* 						priced articles.
*/
public class PriceCalculator1 {

	/**
	* Represents the last time the price was calculated (time of the
	* last order).
	*/
	private Calendar lastOrderTimeStamp;

	/**
	* This method calculates the price of variable price articles
	* depending of the amount sold since the last calculation of
	* the price, the expected standard amount of sales during
	* the interval between two calculations, the standard and
	* current prices and the upper and lower bounds of the price.
	*
	* @pre 			0.0 < lowerBoundary < upperBoundary AND
	* 				amountSold >= 0 AND
	* 				standardAmountSold >= 0
	* @post 		lowerBoundary <= return <= upperBoundary
	* @invariant 	lowerBoundary@POST == lowerBoundary@PRE
	* @invariant 	upperBoundary@POST == upperBoundary@PRE
	* @invariant 	standardPrice@POST == standardPrice@PRE
	* @invariant 	standardAmountSold@POST == standardAmountSold@PRE
	* @param 		standardPrice: a float representation of the base
	* 				price of an article
	* @param 		currentPrice: a float representation of the current
	* 				selling price of an article
	* @param 		lowerBoundary: a float representation of the lowest
	*				possible price of an article
	* @param 		upperBoundary: a float representation of the highest
	* 				possible price of an article
	* @param 		amountSold: an int representation of the amount sold
	* 				since the last calculation of the price. If amountSold=0,
	* 				the price rises.
	* @param 		standardAmountPerHour: an float representation of the
	* 				expected sales during one hour: Take the number of the customer-mail,
	* 				devide it by the amount of houres which are in this period, and devide
	* 				again by the amount of products of the category (from the menucard)
	* @param 		timeStamp: a Calendar representation of the time the
	* 				article was ordered. Isn't needed in this algorithmus, so
	* 				timeStamp can be set "null"!
	* @return 		a float representation of the calculated new price
	*/
	public float calculatePrice( 	float standardPrice,
									float currentPrice,
									float lowerBoundary,
									float upperBoundary,
									int amountSold,
									float standardAmountPerHour,
									Calendar timeStamp) {
		
		float minTillMinPrice = 90f;  // time (in minutes) from standardPrice to lowerBoundary (when the tradeble product is regularly ordered)
		float minTillNormPrice = 45f; // time (in minutes) from lower/upperBoundary to standardPrice (when the tradeble product is regularly ordered)
		float minTillMaxPrice = 90f;  // time (in minutes) form standardPrice to upperBoundary (when the tradeble product is regularly ordered)
		float minRegularOrder = 20f;  // time (in minutes) between to orders (when the tradeble product is regularly ordered)
		float minTillPriceRise = 5f;  // time (in minutes) between to tests wether the tradeble product was orderd
		
		if (amountSold == 0) {		// Price goes up (if no products are ordered)			
			float reachTime = 0; // reachTime represents minTillMax/NormPrice, depending on the price
			float priceDiff = 0; // difference between standardPrice and lower/upper boundary			
			if (currentPrice <= standardPrice) {
				reachTime = minTillNormPrice;
				priceDiff = standardPrice - lowerBoundary;					
			}
			if (currentPrice > standardPrice) {
				reachTime = minTillMaxPrice;
				priceDiff = upperBoundary - standardPrice;
			}
			// Amount pricerises till ReachPrice will be reached
			float amountPriceRisesTillReachPrice = (reachTime/minTillPriceRise);
			//System.out.println("amountPriceRisesTillReachPrice: "+amountPriceRisesTillReachPrice);
			
			// total pricerise if nothing was ordered
			float totalPriceRise = (priceDiff/amountPriceRisesTillReachPrice);
			//System.out.println("totalPriceRise: "+totalPriceRise);
			
			//calculate new price
			currentPrice = currentPrice + totalPriceRise;
			if (currentPrice>upperBoundary) {   // currentPrice has to be <= upperBoundary
				currentPrice = upperBoundary;			
			}
		}
		
		if (amountSold > 0) { // Price goes down			
			float reachTime = 0; // reachTime represents minTillMin/NormPrice, depending on the price
			float priceDiff = 0; // difference between standardPrice and lower/upper boundary			
			if (currentPrice <= standardPrice) {
				reachTime = minTillMinPrice;
				priceDiff = standardPrice - lowerBoundary;					
			}
			if (currentPrice > standardPrice) {
				reachTime = minTillNormPrice;
				priceDiff = upperBoundary - standardPrice;
			}			
			
			/*
			// Variant 1: generates higher prices than variant 2
			float amountSaledProductsInReachTime = ((standardAmountPerHour/60)*90);
			System.out.println("Anz. zu verkaufende Prod bis auf Min: "+amountSaledProductsInReachTime);
			float priceFallperProduct = (priceDiff/amountSaledProductsInReachTime);
			System.out.println("PriceFallPerProduct: "+priceFallperProduct);
			*/			
			
			
			// Variant 2: generates lower prices than variant 1
			// when the tradeble product is regularly ordered: Amount of orders till reachPrice is reached
			float amountOrdersTillReachPrice = reachTime/minRegularOrder;
			//System.out.println("amountOrdersTillMin: Anz. Best. bis MinPreis erreicht: "+amountOrdersTillReachPrice);
			
			// Amount sold products till reachPrice is reached
			float standardAmountPerMinTillReachPrice = ((standardAmountPerHour/60)*reachTime);
			//System.out.println("standardAmountPerMinTillMin: standard-Anz. verkaufte Produkte in 90min : "+standardAmountPerMinTillMin);
			
			// Amount of products per order
			float amountPerOrder = (standardAmountPerMinTillReachPrice / amountOrdersTillReachPrice);
			//System.out.println("amountPerOrder : "+amountPerOrder);
			
			// price-fall per order
			float priceFallperOrder = ((priceDiff) / amountOrdersTillReachPrice);
			//System.out.println("priceFallperOrder : "+priceFallperOrder);
			
			// price-fall per product
			float priceFallperProduct = (priceFallperOrder / amountPerOrder);
			// variant 2 ends here
						
			
			//System.out.println("priceFallperProduct : "+priceFallperProduct);
						
			// total price-fall
			float totalPriceFall = ((amountSold * priceFallperProduct)*0.2f);
			//System.out.println("totalPriceFall : "+totalPriceFall);
			
			// calculate new price
			currentPrice = currentPrice - totalPriceFall;
			if (currentPrice<lowerBoundary) {  // currentPrice has to be <= lowerBoundary
				currentPrice = lowerBoundary;
			}
		}		
		
		lastOrderTimeStamp = timeStamp;
		
		return currentPrice;		
	}
}
