package sopra;
import java.util.*;

/**
 * 	bQm
 */

/**
 * This class contains methods and attributes needed to
 *calculate the current price of variable priced articles.
 *The price is calculated on each order
 *
 *@author Team Sopra, Amancio Bouza
 *@copyright 2004-24-3 RG First Version
 *@version 2004-24-3 RG 1.0
 *         2004-06-03 RG 2.0
 *@responsibilities This class calculates prices for variable priced articles
 */

public class PriceCalculator2 {

	/**
	 * Represents the last time the price was calculated (time of the
	 * last order).
	 */
	private Calendar lastOrderTimeStamp;
	/**
	  *Representes the current timestamp of the current order.
	  */
	private Calendar currentOrderTimeStamp;

	/**
	  *Different daytimes
	  */
	public static final int DT_MORNING = 0;
	public static final int DT_NOON = 1;
	public static final int DT_AFTERNOON = 2;
	public static final int DT_EVENING = 3;
	public static final int DT_NIGHT = 4;
	/**
	  *Elasticity for 5 different daytimes.
	  */
	private float[] Elasticity = { -1f, -1f, -1f, -1f, -1f };
	/**
	  *counter for total sold products for each daytime.
	  */
	private int[] totalSold = { 0, 0, 0, 0, 0 };
	/**
	  *default price for each daytime.
	  */
	private float[] defaultPrice = { 0f, 0f, 0f, 0f, 0f };
	/**
	  *maximum price where the calculated amount of sold products is zero.
	  */
	private float[] maxPrice = { 1f, 1f, 1f, 1f, 1f };
	/**
	  *default amount per hour of sold products for each daytime.
	  */
	private float[] defaultAmount = { 0f, 0f, 0f, 0f, 0f };

	/**
	 * This method calculates the price of variable price articles
	 * depending of the amount sold since the last calculation of
	 * the price, the expected standard amount of sales during
	 * the interval between two calculations, the standard and
	 * current prices and the upper and lower bounds of the price.
	 *
	 * @pre 0.0 < lowerBoundary < upperBoundary AND
	 * amountSold >= 0 AND
	 * standardAmountSold >= 0
	 * @post lowerBoundary <= return <= upperBoundary
	 * @invariant lowerBoundary@POST == lowerBoundary@PRE
	 * @invariant upperBoundary@POST == upperBoundary@PRE
	 * @invariant standardPrice@POST == standardPrice@PRE
	 * @invariant standardAmountSold@POST == standardAmountSold@PRE
	 * @param standardPrice: a float representation of the base
	 * price of an article Softwarepraktikum Prof. Dr. M. Glinz
	 * Sommersemester 2004 Robert Stoyan AnleitungVersuch06.doc - 6 -
	 * @param currentPrice: a float representation of the current
	 * selling price of an article
	 * @param lowerBoundary: a float representation of the lowest
	 * possible price of an article
	 * @param upperBoundary: a float representation of the highest
	 * possible price of an article
	 * @param amountSold: an int representation of the amount sold
	 * since the last calculation of the price
	 * @param standardAmountPerHour: an float representation of the
	 * expected sales during one hour
	 * @param timeStamp: a Calendar representation of the time the
	 * article was ordered
	 * @return a float representation of the calculated new price
	 */

	public float calculatePrice(
		float standardPrice,
		float currentPrice,
		float lowerBoundary,
		float upperBoundary,
		int amountSold,
		float standardAmountPerHour,
		Calendar timeStamp) {
		//Initialisation
		float newCurrentPrice = currentPrice;
		currentOrderTimeStamp = timeStamp;

		if (amountSold > 0) {
			/*
			* calculate new decreased price for the product coming from an new order.
			*/

			//Initialisation
			init(
				standardPrice,
				currentPrice,
				lowerBoundary,
				upperBoundary,
				amountSold,
				standardAmountPerHour,
				timeStamp);

			//calculate new current price depending the optimal price.
			float optPrice = getOptimalPrice();
			for (int i = 0; i < amountSold; i++) {
				if (currentPrice >= optPrice) {
					newCurrentPrice = Math.max(lowerBoundary, Math.min(upperBoundary, newCurrentPrice - 0.052f));
				} else {
					newCurrentPrice = Math.max(lowerBoundary, Math.min(upperBoundary, newCurrentPrice - 0.012f));
				}
			}

			//Finish
			lastOrderTimeStamp = timeStamp;
			if (newCurrentPrice * 0 != 0) {
				for (int i = 0; i < amountSold; i++) {
					newCurrentPrice = Math.max(lowerBoundary, Math.min(upperBoundary, newCurrentPrice - 0.012f));
				}
			}

			System.out.println("c$:\t" + newCurrentPrice);
			
		} else {
			/*
			* calculate new price 
			*/

			//calculate new current price depending the optimal price.    
			float optPrice = getOptimalPrice();
			if (currentPrice >= optPrice) {
				newCurrentPrice = Math.max(lowerBoundary, Math.min(upperBoundary, currentPrice + 0.012f));
			} else {
				newCurrentPrice = Math.max(lowerBoundary, Math.min(upperBoundary, currentPrice + 0.052f));
			}

			if (newCurrentPrice * 0 != 0) {
				newCurrentPrice = Math.max(lowerBoundary, Math.min(upperBoundary, currentPrice + 0.012f));
			}
		}

		finish(amountSold);
		return newCurrentPrice;
	}

	/**
		*Finishs the calculation process
		*@param amountSold
		*@post: totalSold increased with amountSold
		*/
	private void finish(int amountSold) {
		totalSold[getDayTime()] += amountSold;
	}

	/**
	  *calculates the default price, default amount per hour and the elasticity
	* @param standardPrice
	* @param currentPrice
	* @param lowerBoundary
	* @param upperBoundary
	* @param amountSold
	* @param standardAmountPerSecond
	*/
	private void init(
		float standardPrice,
		float currentPrice,
		float lowerBoundary,
		float upperBoundary,
		int amountSold,
		float standardAmountPerHour,
		Calendar timeStamp) {

		calcDefaultPrice(standardPrice);
		calcDefaultAmountPerHour(standardAmountPerHour);
		calcElasticity(currentPrice, getCurrentAmountPerHour(amountSold), amountSold);
	}

	/**
	  *Calculcates the optimal price. The following function is used to get the optimal price.<br><br>
	  *(Price := x-Axis, Amount per Hour := y-Axis)<br>
	  *f(x) = x * ( (xMax-x) * tan(a) ) = x * (tan(a)*xMax - tan(a)*x)
	  *= (tan(a)*xMax)*x - (tan(a))*x^2<br>
	  *d/dx f(x)= tan(a)*xMax - (2*tan(a))*x<br>
	  *Maximum at d/dx f(x) = 0 = xmax * tan(alpha) - 2*tan(alpha)*x<br>
	  *=>x = xmax*tan(alpha) / ( 2*tan(alpha)) = xmax / 2
	  *
	  *tan(alpha) = a / b = -Elasticity => Maximum @ x = xmax / (2 * Elasticity)<br>
	  *@return float current Optimal Price calculated what should maximize the money won.
	  *@pre - 
	  *@post maxPrice will be new calculated
	  */
	private float getOptimalPrice() {
		float optPrice = calcMaximalPrice() / 2;

		return optPrice;
	}

	/**
	  *Calulates the maximum price, where the amount of sold products is zero.<br><br>
	  *(Price := x, Amount Per Hour := y-Axis)<br>
	  *y = a*x + b<br>
	  *=> xmax: y = 0 = a*x + b <=> x = -b / a
	  *@pre Elasticity[getDayTime()] != 0
	  *@post maximum price will be new calculated and stored in maxPrice.
	  */
	private float calcMaximalPrice() {
		float maxPrice = (-1f * defaultAmount[getDayTime()]) / Elasticity[getDayTime()] + defaultPrice[getDayTime()];
		return maxPrice;
	}

	/**
	  *Calculates the Elasticity.
	  *@param currentPrice current price.
	  *@param currentAmount current amount per second
	  *@param amountSold number of sold products
	  *@pre currentPrice != defaultPrice[getDayTime()]
	  *@post Elastiicty is new calculated 
	  * and the new value is stored at the index getDayTime().
	  */
	private void calcElasticity(float currentPrice, float currentAmountPerHour, int amountSold) {
		//Calc current elasticity
		float height = currentAmountPerHour - defaultAmount[getDayTime()];
		float width = currentPrice - defaultPrice[getDayTime()];
		float currentElasticity = height / width;

		if ((currentElasticity * 0) != 0) {
			currentElasticity = -1f;
		}

		//calc total elasticity
		Elasticity[getDayTime()] =
			(currentElasticity * (float) amountSold + Elasticity[getDayTime()] * (float) totalSold[getDayTime()])
				/ (float) (amountSold + totalSold[getDayTime()]);
	}

	/**
	  *Calulates the default price.
	  *@param standardPrice standard price.
	  *@post defaultPrice will be new calculated and stored at the index getDayTime()
	  *@see getDayTime()
	  */
	private void calcDefaultPrice(float standardPrice) {
		/*float newDefaultPrice = (defaultPrice[getDayTime()] * totalSold[getDayTime()]
														+ currentPrice * amountSold)
													/ (amountSold + totalSold[getDayTime()]);*/
		defaultPrice[getDayTime()] = standardPrice;
	}

	/**
	  *Calculates the default amount per hour.
	  *@param standardAmountPerHour standard amount of products per hour
	  *@post default amount per of products per seconds will be new calculated 
	  * and stored in defaultAmount at the index getDayTime().
	  *@see #getDayTime()
	  */
	private void calcDefaultAmountPerHour(float standardAmountPerHour) {
		/*float newDefaultAmount = (defaultAmount[getDayTime()] 
									  * (float) totalSold[getDayTime()]
									+ currentAmount 
									  * (float) amountSold)
									/ (float) (amountSold + totalSold[getDayTime()]);*/
		
		defaultAmount[getDayTime()] = standardAmountPerHour;
	}

	/**
	  *Getts the current amount per hour.
	  *@param amountSold amount of sold products
	  *@pre currenOrderTimeStamp != lastOrderTimeStamp  
	  */
	private float getCurrentAmountPerHour(int amountSold) {
		float periodInSecond =
			((currentOrderTimeStamp.getTimeInMillis()) - (lastOrderTimeStamp.getTimeInMillis())) / 1000f;
		float periodInHour = periodInSecond / (60f * 60f);

		float currentAmountPerHour = (float) (amountSold / periodInHour);

		return currentAmountPerHour;
	}

	/**
	  *Gets the identifier of the day time. the day is breaken in the following parts:<br>
	  *Morning: (6, 11)<br>
	  *Noon: [11, 14)<br>
	  *Afternoon: [14, 17)<br>
	  *Evening: [17, 21)
	  *Night: [21, 6]<br>
	  *@return int identifier of the day time
	  *@pre currentOrderTimeStamp != null
	  */
	private int getDayTime() {
		if ((currentOrderTimeStamp.get(Calendar.HOUR_OF_DAY) > 6)
			& (currentOrderTimeStamp.get(Calendar.HOUR_OF_DAY) < 11)) {
			return PriceCalculator2.DT_MORNING;
		} else if (currentOrderTimeStamp.get(Calendar.HOUR_OF_DAY) < 14) {
			return PriceCalculator2.DT_NOON;
		} else if (currentOrderTimeStamp.get(Calendar.HOUR_OF_DAY) < 17) {
			return PriceCalculator2.DT_AFTERNOON;
		} else if (currentOrderTimeStamp.get(Calendar.HOUR_OF_DAY) < 21) {
			return PriceCalculator2.DT_EVENING;
		} else {
			return PriceCalculator2.DT_NIGHT;
		}
	}

	/**
	  *Default Constructor
	  *@param startTime Timestamp of the start of the application
	  *@pre startTime != null
	  *@post lastOrderTimeStamp = startTime
	  */
	public PriceCalculator2(Calendar startTime) {
		lastOrderTimeStamp = startTime;
	}
	
	public PriceCalculator2() {
	}
}
