package sopra;

/**
 * A-TEAM
 */

import java.util.Calendar;
/**
 * This class contains methods and attributes needed to
 *calculate the current price of variable priced articles.
 *The price is calculated on each order
 *
 *@author Team Sopra
 *@copyright 2004-24-3 RG First Version
 *@version 2004-24-3 RG 1.0
 *@responsibilities This class calculates prices for variable priced articles
 */

public class PriceCalculator3 {

	/**
	 * Represents the last time the price was calculated (time of the
	 * last order).
	 */
	private Calendar lastOrderTimeStamp;
	/**
	 * This method calculates the price of variable price articles
	 * depending of the amount sold since the last calculation of
	 * the price, the expected standard amount of sales during
	 * the interval between two calculations, the standard and
	 * current prices and the upper and lower bounds of the price.
	 *
	 * @pre 0.0 < lowerBoundary < upperBoundary AND
	 * amountSold >= 0 AND
	 * standardAmountSold >= 0
	 * @post lowerBoundary <= return <= upperBoundary
	 * @invariant lowerBoundary@POST == lowerBoundary@PRE
	 * @invariant upperBoundary@POST == upperBoundary@PRE
	 * @invariant standardPrice@POST == standardPrice@PRE
	 * @invariant standardAmountSold@POST == standardAmountSold@PRE
	 * @param standardPrice: a float representation of the base
	 * price of an article Softwarepraktikum Prof. Dr. M. Glinz
	 * Sommersemester 2004 Robert Stoyan AnleitungVersuch06.doc - 6 -
	 * @param currentPrice: a float representation of the current
	 * selling price of an article
	 * @param lowerBoundary: a float representation of the lowest
	 * possible price of an article
	 * @param upperBoundary: a float representation of the highest
	 * possible price of an article
	 * @param amountSold: an int representation of the amount sold
	 * since the last calculation of the price
	 * @param standardAmountPerHour: an float representation of the
	 * expected sales during one hour
	 * @param timeStamp: a Calendar representation of the time the
	 * article was ordered
	 * @return a float representation of the calculated new price
	 */

	public float calculatePrice(
		float standardPrice,
		float currentPrice,
		float lowerBoundary,
		float upperBoundary,
		int amountSold,
		float standardAmountPerHour,
		Calendar timeStamp) {
		float price;
		float factor;
		standardAmountPerHour = standardAmountPerHour / 6; // da wir alle 10 min abrechen zZ
		if (standardAmountPerHour <= 4) {
			factor = 0.1f;
		} else if (standardAmountPerHour >= 20) {
			factor = 0.02f;
		} else {
			factor = 0.05f;
		}

		if ((standardAmountPerHour - amountSold) * factor >= ((upperBoundary - lowerBoundary) / 20))
			// wenn steigung zu stark
			{
			price = currentPrice + ((upperBoundary - lowerBoundary) / 20); // maximal steigung
		} else if ((standardAmountPerHour - amountSold) * ((-1) * factor) >= ((upperBoundary - lowerBoundary) / 18))
			// wenn steigung zu negativ
			{
			price = currentPrice - ((upperBoundary - lowerBoundary) / 18);
			//maximal preisfall - hier kann der preis schneller fallen als steigen. vielleicht gut wegen der preiselatizitt?
		} else {
			price = currentPrice + (standardAmountPerHour - amountSold) * factor;
		} //normale quote, muss angeglichen werden nach wirtwerten

		if (price > upperBoundary) {
			price = upperBoundary;
		}
		if (price < lowerBoundary) {
			price = lowerBoundary;
		}
		//price = PersCashUpGUI.round(price);
		return price;
	}
}
