package sopra;

/**
 * RPG
 */

import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.Observable;
import java.util.Vector;

/**
 * This class contains methods and attributes needed to
 * calculate the current price of variable priced articles.
 * The price is calculated on each order.
 * 
 * @author     Team Sopra // Roman Wieser
 * @copyright  Departement of Computer Science, Requirement
 * @history    2004-24-03 RG First Version
 * @version    2004-24-05 RG 1.0 beta 1 test
 * 
 * @responsibilities This class calculates prices for 
 *                   variable priced articles.
 */

public class PriceCalculator5 extends Observable {
	/**
	 * Represents the last time the price was calculated (time
	 * of the last order)
	 *//////actually nerver used....done by sopra-team
	private Calendar lastOrderTimeStamp;
	
	//the type to save to DB
	private static float money;
	
	//orders which arent calculated for dispay
	private static Vector uncalcOrders;
	
	//products on display
	//public static Vector checkedProducts = server.SqlConnection.getCheckedProducts();
	
	//needed to get actual hour
	private static Calendar now = new GregorianCalendar();
	
	//price chrash
	public static boolean crash = false;
	
	//categories
	private static int[][] averageAmount= { 	{15, 10, 90}, 		//Hochproz
												{50, 200, 800},		//Bier
												{40, 10, 40},		//Wein
												{200, 200, 200},	//Non-Alk
												{90, 10, 50}};		//Food
	
	//test wheter simulation or real time
	public static boolean real=true;
	
	/**
	 * This method calculates the price of variable price articles
	 * depending of the amount sold since the last calculation of
	 * the price, the expected standart amount of sales during
	 * the interval between two calculations, the standart and 
	 * current prices and the upper and lower bounds of the price.
	 * 
	 * @pre		0.0 < lowerBoundary < upperBoundary AND
	 * 			amountSold >= 0 AND 
	 * 			standartAmountSold >=0
	 * @post	lowerBoundary <= return <= upperBoundary
	 * @invariant lowerBoundary@POST == lowerBoundary@PRE
	 * @invariant upperBoundary@POST == upperBoundary@PRE
	 * @invariant standartPrice@POST == standartPrice@PRE
	 * @invariant standartAmountSold@POST == standartAmountSold@PRE
	 * @param	standartPrice: a float representation of the base 
	 * 				price of an article
	 * @param	currentPrice: a float representation of the current
	 * 				selling price of an article
	 * @param	lowerBoundary: a float representation of the lowest
	 * 				possible price of an article
	 * @param	upperBoundary: a float representation of the highest
	 * 				possible price of an article
	 * @param	amountSold: an int representation of the amount solde 
	 * 				since the last calculation of the price
	 * @param	standartAmountPerHour: a float representation of the
	 * 				expected sales during one hour
	 * @param	timeStamp: a Calender representation of the time the
	 * 				article was ordered
	 * @return  a float representation of the calculated new price
	 */
	public float calculatePrice(	float standartPrice,
									float currentPrice,
									float lowerBoundary,
									float upperBoundary,
									int amountSold,
									float standartAmountPerHour,
									Calendar timeStamp){
								
		float newPrice=0;
		
		float upperDifference = upperBoundary-standartPrice;
		float lowerDifference = standartPrice-lowerBoundary;
		
		if(crash) {
			//random value
			double rmd = Math.random();
			//random value between 0 and 0.2
			double diff = Math.random() / 5;
			if(rmd < 0.75){
				return lowerBoundary;
			}
			else {
				return (float)(lowerBoundary*(1+diff));
			}
		}

		boolean test;
		if(currentPrice<standartPrice){
			test=true;
		}
		else test=false;
		
		switch(caseChooser(amountSold,standartAmountPerHour)){
			case 1:
				
				float differenceAmount = standartAmountPerHour - amountSold;
				float qoutient = differenceAmount/standartAmountPerHour;
			
				//calc the heigth of the actual price level
				float currentDif= upperBoundary-currentPrice;
			
				//the lower the price, the harder the rise :-)
				float hardNess=1-currentDif/upperBoundary;
				
				//30% hardness+ 70%qoutient = change of actuall price
				float changer= 0.3f*hardNess+0.7f*qoutient;
				
				//different change due to different bounds
				if(test){
					newPrice= currentPrice+ upperDifference *changer;
				}else{
					newPrice= currentPrice+ lowerDifference *changer;
				}
				break;
		
			case 2:
				float differenceAmount1 = amountSold - standartAmountPerHour;
				float quotient = differenceAmount1/standartAmountPerHour;
				
				if(quotient>1){
					quotient=1.5f;
				}
				
				if(currentPrice==lowerBoundary){
					newPrice= lowerBoundary+lowerDifference/3;
				}
				else{
					if(test){
						newPrice= currentPrice- upperDifference * quotient;
					}else{
						newPrice= currentPrice- lowerDifference * quotient;
					}
				}
				break;
			
			case 3:
					
					//test, that price is not alway on top 3 cases:
					//				- low price products
					//				- medium pp
					//				- high pp
					if(upperDifference<3 && currentPrice+(Math.random()/2)>upperBoundary){
						return upperBoundary-(float)Math.random();
					}	
					if(upperDifference>=3 &&upperDifference<7 && currentPrice+Math.random()>upperBoundary){
						return upperBoundary-(float)Math.random()*4;
					}
					
					if(upperDifference>=7 && currentPrice+Math.random()>upperBoundary){
						return upperBoundary-(float)Math.random()*6;
					}
					
					//increase price 1/3 of max-cur differencet
					float diff = upperBoundary - currentPrice;
					
					float change=diff/2*(float)Math.random();
					
					newPrice = currentPrice+change;

					break;	
			}
			
		
		
		
		//dont let the new Price out of bounds
		if(newPrice<lowerBoundary){
			newPrice= lowerBoundary;
		}
		if(newPrice>upperBoundary){
			newPrice= upperBoundary;
		}
		
  
		double xr = Math.round(newPrice*100.0)/100.0;   
  
		return (float)xr;
	}
	
	/**chooses the actual case for calculation
	 * 
	 * @param int amountSold
	 * @param float standartAmountPerHour
	 * @return
	 */
	private int caseChooser(int amountSold, float standartAmountPerHour ){
		if(standartAmountPerHour>= amountSold && amountSold!=0) return 1;
		if(standartAmountPerHour< amountSold&& amountSold!=0 ) return 2;
		if(amountSold==0) return 3;
		
		return -1;
	}
	
	/**sets all newPrices to DB and notifies Display
	 * @pre
	 * @post
	 * @param
	 * @return
	 */
	/*
	public void getAllStuff(){
		
		uncalcOrders = server.SqlConnection.readUncalcOrders();
		
		
		for(int i=0; i<checkedProducts.size(); i++) {
			
			Integer id = (Integer)((Vector)checkedProducts.get(i)).get(0);
			Float min = (Float)((Vector)checkedProducts.get(i)).get(3);
			Float max = (Float)((Vector)checkedProducts.get(i)).get(4);
			Float std = (Float)((Vector)checkedProducts.get(i)).get(2);
			
			Float cur = (Float)((Vector)checkedProducts.get(i)).get(7);
			
			
			Integer category=(Integer)((Vector)checkedProducts.get(i)).get(5);
			
			
			money = calculatePrice(	std.floatValue(),
									cur.floatValue(),
									min.floatValue(),
									max.floatValue(),
									getSoldAmount(id.intValue()),
									getAverageAmountPerHour(category.intValue()),
									now);
			
			server.SqlConnection.editCurrentPrice(id.intValue(),money);
		}
		
		setChanged();
		notifyObservers();
	}*/
	
}