package sopra;

/**
 * 	COFFEE.COM
 */

import java.util.Calendar;
import java.util.GregorianCalendar;
/**
 * This class contains methods and attributes needed to
 *calculate the current price of variable priced articles.
 *The price is calculated on each order
 *
 *@author Team Sopra
 *@copyright 2004-24-3 RG First Version
 *@version 2004-24-3 RG 1.0
 *		   2004-06-4 TW 1.1 Methode ausprogrammiert	
 *		   2004-06-12 TW 1.2 Anpassung bei leerem lastOrderTimeStamp
 *@responsibilities This class calculates prices for variable priced articles
 */

public class PriceCalculator6 {

	/**
	 * Represents the last time the price was calculated (time of the
	 * last order).
	 */
		private Calendar lastOrderTimeStamp;
		
	/**
	 * This method calculates the price of variable price articles
	 * depending of the amount sold since the last calculation of
	 * the price, the expected standard amount of sales during
	 * the interval between two calculations, the standard and
	 * current prices and the upper and lower bounds of the price.
	 *  
	 * @pre 0.0 < lowerBoundary < upperBoundary AND
	 * amountSold >= 0 AND
	 * standardAmountSold >= 0
	 * @post lowerBoundary <= return <= upperBoundary
	 * @invariant lowerBoundary@POST == lowerBoundary@PRE
	 * @invariant upperBoundary@POST == upperBoundary@PRE
	 * @invariant standardPrice@POST == standardPrice@PRE
	 * @invariant standardAmountSold@POST == standardAmountSold@PRE
	 * @param standardPrice: a float representation of the base
	 * price of an article Softwarepraktikum Prof. Dr. M. Glinz
	 * Sommersemester 2004 Robert Stoyan AnleitungVersuch06.doc - 6 -
	 * @param currentPrice: a float representation of the current
	 * selling price of an article
	 * @param lowerBoundary: a float representation of the lowest
	 * possible price of an article
	 * @param upperBoundary: a float representation of the highest
	 * possible price of an article
	 * @param amountSold: an int representation of the amount sold
	 * since the last calculation of the price
	 * @param standardAmountPerHour: an float representation of the
	 * expected sales during one hour
	 * @param timeStamp: a Calendar representation of the time the
	 * article was ordered
	 * @return a float representation of the calculated new price
	 */
	public float calculatePrice( float standardPrice,
								 float currentPrice,
								 float lowerBoundary,
								 float upperBoundary,
								 int amountSold,
								 float standardAmountPerHour,
								 Calendar timeStamp){
								 	
	 	
		double neuerPreis = 0;
		double f = 0;			
		//Fr ersten Aufruf dieser Methode kann der lastOrderTimeStamp nicht benutzt
		//werden, deshalb eine relative Preisnderung
		if (lastOrderTimeStamp==null){
			if (amountSold < standardAmountPerHour){
				neuerPreis = currentPrice + currentPrice/20;			
			}
			if (amountSold > standardAmountPerHour){
				neuerPreis = currentPrice - currentPrice/20;			
			}
			if (amountSold == standardAmountPerHour){
				neuerPreis = currentPrice;			
			}
		}
		else{					 	
			double standardAmountPerInterval = 0;

			double intervall = timeStamp.getTimeInMillis() - lastOrderTimeStamp.getTimeInMillis() ;
			
			double faktor = intervall / (60*60*1000);
			
			standardAmountPerInterval = standardAmountPerHour * faktor;		
			
			if (amountSold  == 0) neuerPreis = currentPrice + standardPrice/10;			
			else {			 						 				 	
				if (amountSold < standardAmountPerInterval){
					//f = Prozente ber Durchschnitt
					f = (standardAmountPerInterval-amountSold)/ standardAmountPerInterval;
					
					//if (f > 1)	neuerPreis = currentPrice + (upperBoundary - currentPrice)*f;	
					//else		neuerPreis = currentPrice + (upperBoundary - currentPrice)*f;
					if (f < 0.25)	neuerPreis = currentPrice + (upperBoundary - currentPrice)*0.1;
					else if (f < 0.5)	neuerPreis = currentPrice + (upperBoundary - currentPrice)*0.2;
					else if (f < 0.75)	neuerPreis = currentPrice + (upperBoundary - currentPrice)*0.3;
					else if (f < 1)	neuerPreis = currentPrice + (upperBoundary - currentPrice)*0.4;
					else if (f < 1.5)	neuerPreis = currentPrice + (upperBoundary - currentPrice)*0.5;
					else  neuerPreis = currentPrice - (upperBoundary - currentPrice)*0.7;
				}
				if (amountSold > standardAmountPerInterval){
					//f = Prozente ber Durchschnitt 
					f = (amountSold-standardAmountPerInterval) / standardAmountPerInterval;
					
					//if (f > 1)	neuerPreis = currentPrice - (currentPrice - lowerBoundary)*f;	
					//else		neuerPreis = currentPrice - (currentPrice - lowerBoundary)*f;
					if (f < 0.25)	neuerPreis = currentPrice - (currentPrice - lowerBoundary)*0.1;
					else if (f < 0.5)	neuerPreis = currentPrice - (currentPrice - lowerBoundary)*0.2;
					else if (f < 0.75)	neuerPreis = currentPrice - (currentPrice - lowerBoundary)*0.3;
					else if (f < 1)	neuerPreis = currentPrice - (currentPrice - lowerBoundary)*0.4;
					else if (f < 1.5)	neuerPreis = currentPrice - (currentPrice - lowerBoundary)*0.5;
					else  neuerPreis = currentPrice - (currentPrice - lowerBoundary)*0.7;
				}
				if (amountSold == standardAmountPerInterval) neuerPreis = currentPrice;
			}
		}
		
		if (neuerPreis > upperBoundary) neuerPreis = upperBoundary;
		if (neuerPreis < lowerBoundary) neuerPreis = lowerBoundary;	
		return (float)neuerPreis;
	}
	
	/**
	 Setzt von aussen den lastOrderTimeStamp auf den neuen Wert. Wird hier jeweils 
	 nach einer Berechnungsserie verndert
	 @pre 	eine Berechnungsserie aller variablen Produkte vorbei
	 @post	neue Zeit gesetzt
	 @param Calendar Timestamp, reprsentiert die neu zu setzende Zeit
	 */
	public void setTimeStamp(Calendar timeStamp){
		if (lastOrderTimeStamp==null){
			lastOrderTimeStamp = new GregorianCalendar();
		}
		lastOrderTimeStamp.set(Calendar.YEAR, timeStamp.get(Calendar.YEAR));
		lastOrderTimeStamp.set(Calendar.MONTH, timeStamp.get(Calendar.MONTH));
		lastOrderTimeStamp.set(Calendar.DAY_OF_MONTH, timeStamp.get(Calendar.DAY_OF_MONTH));		
		lastOrderTimeStamp.set(Calendar.HOUR_OF_DAY, timeStamp.get(Calendar.HOUR_OF_DAY));		
		lastOrderTimeStamp.set(Calendar.MINUTE, timeStamp.get(Calendar.MINUTE));		
		lastOrderTimeStamp.set(Calendar.SECOND, timeStamp.get(Calendar.SECOND));
		lastOrderTimeStamp.set(Calendar.MILLISECOND, timeStamp.get(Calendar.MILLISECOND));	
	}
								

}

