package sopra;

/**
 * Sopla-null-viel
 */

import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.Hashtable;
import java.util.Random;
/**
 * This class contains methods and attributes needed to
 *calculate the current price of variable priced articles.
 *The price is calculated on each order
 *
 *@author Team Sopra
 *@copyright 2004-22-6 EB Final Version
 *@version 2004-24-3 RG 1.0
 *         2004-22-6 EB Final Version
 *@responsibilities This class calculates prices for variable priced articles
 */

public class PriceCalculator8
{

	/**
	 * Represents the last time the price was calculated (time of the
	 * last order).
	 */
	private Calendar lastOrderTimeStamp;
	/**
	 * This method calculates the price of variable price articles
	 * depending of the amount sold since the last calculation of
	 * the price, the expected standard amount of sales during
	 * the interval between two calculations, the standard and
	 * current prices and the upper and lower bounds of the price.
	 *
	 * @pre 0.0 < lowerBoundary < upperBoundary AND
	 * amountSold >= 0 AND
	 * standardAmountSold >= 0
	 * @post lowerBoundary <= return <= upperBoundary
	 * @invariant lowerBoundary@POST == lowerBoundary@PRE
	 * @invariant upperBoundary@POST == upperBoundary@PRE
	 * @invariant standardPrice@POST == standardPrice@PRE
	 * @invariant standardAmountSold@POST == standardAmountSold@PRE
	 * @param standardPrice: a float representation of the base
	 * price of an article Softwarepraktikum Prof. Dr. M. Glinz
	 * Sommersemester 2004 Robert Stoyan AnleitungVersuch06.doc - 6 -
	 * @param currentPrice: a float representation of the current
	 * selling price of an article
	 * @param lowerBoundary: a float representation of the lowest
	 * possible price of an article
	 * @param upperBoundary: a float representation of the highest
	 * possible price of an article
	 * @param amountSold: an int representation of the amount sold
	 * since the last calculation of the price
	 * @param standardAmountPerHour: an float representation of the
	 * expected sales during one hour
	 * @param timeStamp: a Calendar representation of the time the
	 * article was ordered
	 * @return a float representation of the calculated new price
	 */

	private Hashtable oldData = null;

	public PriceCalculator8()
	{
		oldData = new Hashtable();
	}

	public float calculatePrice(
		float standardPrice,
		float currentPrice,
		float lowerBoundary,
		float upperBoundary,
		int amountSold,
		float standardAmountPerHour,
		Calendar timeStamp)
	{
		float newPrice = currentPrice;
		float standardExchange = standardPrice * standardAmountPerHour;
		float exchange = currentPrice * amountSold;
		float mult = 1;
		Float hash = new Float(standardPrice + lowerBoundary + upperBoundary);

		if (oldData.containsKey(hash))
		{
			lastOrderTimeStamp = (Calendar)oldData.get(hash);
			long time = timeStamp.getTimeInMillis() - lastOrderTimeStamp.getTimeInMillis();
			double timeInHour = ((double)time / 1000) / 3600;
			amountSold = (int)((double)amountSold * (1/timeInHour));
			oldData.remove(hash);
		}
		else
		{
		  if((((float)amountSold)/standardAmountPerHour) < 0.8){
			amountSold = (int)standardAmountPerHour;
		  }
		}
	oldData.put(hash, timeStamp);

		/* If price falls */
		if (exchange > standardExchange)
		{
			mult = (1 - (standardExchange / exchange)) / 2;
			newPrice = currentPrice - ((currentPrice - lowerBoundary) * mult);
		}
		/* If price goes up */
		else if (exchange < standardExchange)
		{
			mult = (1 - (exchange / standardExchange)) / 2;
			newPrice = currentPrice + ((upperBoundary - currentPrice) * mult);
		}

		if (newPrice > upperBoundary)
		{
			newPrice = upperBoundary;
		}
		else if (newPrice < lowerBoundary)
		{
			newPrice = lowerBoundary;
		}

		return ((float) ((int) ((20 * newPrice) + 0.5))) / 20;
	}

	public static void main(String[] args)
	{
	  PriceCalculator pc = new PriceCalculator();
	  Calendar cal = null;
	  float current = 3.8f;

	  for(int i = 0; i < 1000; i++){
		cal = new GregorianCalendar(1990,1,1,i,0,0);
		current = pc.calculatePrice(4, current, 3, 5, 40 + (new Random()).nextInt(20), 50, cal);
		System.out.println(current);
	  }
	}
}
