package sopra;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.MediaTracker;
import java.awt.Toolkit;
import java.awt.Window;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.net.URL;

public class Splash extends Window {
	// Time to display the splash screen
	final static int TIME_TO_WAIT = 3000;
    
	// Helper variabels
	private Image splashImage;
	private int imgWidth, imgHeight;
	private static final int BORDERSIZE = 10;
	private static final Color BORDERCOLOR = new Color(204,0,0);
	Toolkit tk;
    
	/**
	 *  Create new splash screen an show it
	 *  @param Frame f, String imgName
	 */
	public Splash(Frame f, String imgName) {
		// Get toolkit
		super(f);
		tk = Toolkit.getDefaultToolkit();
        
		// Load the splash image
		splashImage = loadSplashImage(imgName);
        
		// Show the splash screen
		showSplashScreen();
        
		// Add window listener
		f.addWindowListener(new WindowListener());
	}
    
	public Image loadSplashImage(String imgName) {
		MediaTracker tracker = new MediaTracker(this);
		Image result;
        
		URL myURL = getClass().getResource(imgName);
        
		// Trying to load imagex
		result = tk.getImage(myURL);
        	
		// Check if image could be loaded
		if(result == null) { System.out.println("ERROR: Splash screen couldn't be loaded"); }
        
		// Add Image
		tracker.addImage(result, 0);
		try { tracker.waitForAll(); }
		catch (Exception e) { e.printStackTrace(); }
        
		// Get image width and height
		imgWidth = result.getWidth(this);
		imgHeight = result.getHeight(this);
        
		return (result);
	}

	public void showSplashScreen() {
		Dimension screenSize = tk.getScreenSize();
		setBackground(BORDERCOLOR);
        
		// Get dimensions, set bounds
		int w = imgWidth + (BORDERSIZE * 2);
		int h = imgHeight + (BORDERSIZE * 2);
		int x = (screenSize.width - w) /2;
		int y = (screenSize.height - h) /2;
		setBounds(x, y, w, h);
        
		// show splash screen
		setVisible(true);
        
		// Wait TIME_TO_WAIT
		try {
			Thread.sleep(TIME_TO_WAIT);
		} catch(InterruptedException ie){}
	}

	public void paint(Graphics g) {
		g.drawImage(splashImage, BORDERSIZE, BORDERSIZE, imgWidth, imgHeight, this);
	}

	class WindowListener extends WindowAdapter {
		public void windowActivated(WindowEvent we) {
			setVisible(false);
			dispose();
		}
	}
}