package sopra;

import java.net.MalformedURLException;
import java.util.Random;
import javax.swing.JFrame;
import java.applet.Applet;
import java.applet.AudioClip;

/**
 * Runs the test
 */
public class TestRunner {
	/**
	 * Progress message
	 */
	private String progressMessage;

	/**
	 * Remaining time
	 */
	private int remainingTime = 0;

	/**
	 * Boolean for testIsRunning
	 */
	private boolean testIsRunning = false;

	/**
	 * Is done
	 */
	private boolean isDone = false;

	/**
	 * Container for test data
	 */
	private float[] testData = new float[8];

	/**
	 * Constructor
	 */
	public TestRunner() {
		// Set running flag
		this.testIsRunning = true;
	}

	/**
	 * Called from ProgressBarDemo to start the task.
	 */
	public void go() {
		final SwingWorker worker = new SwingWorker() {
			public Object construct() {
				remainingTime = 0;
				progressMessage = null;
				return new ActualTask();
			}
		};
		worker.start();
	}

	/**
	 * Returns the most recent status message, or null
	 * if there is no current status message.
	 */
	public String getMessage() {
		return progressMessage;
	}

	/**
	 * Returns the most recent status message, or null
	 * if there is no current status message.
	 */
	public float[] getTestData() {
		return this.testData;
	}

	public boolean isDone() {
		return this.isDone;
	}

	public boolean isRunning() {
		return this.testIsRunning;
	}

	public void stop() {
		this.testIsRunning = false;
	}

	/**
	 * Shows a splash screen for a specified group
	 * @param pic
	 */
	public void showSplash(String pic) {
		// Play horn sound
		try {
			// Play audio notification
			AudioClip clip = Applet.newAudioClip(TestSystem.soundHorn.toURL());
			clip.play();

		} catch (MalformedURLException e) {
		}

		// Show splash with pic
		Splash splash = new Splash(new JFrame(), pic);

		// Hide splash
		splash.setVisible(false);
		
		// Dispose it
		splash.dispose();
	}

	/**
	 * The actual long running task.  This runs in a SwingWorker thread.
	 */
	class ActualTask {
		ActualTask() {
			// Get timeToTest
			int timeToTest = TestSystem.RUN_TIME * 10;

			// Initialize test object
			CafeTest test = new CafeTest();

			// Loop counting down test time
			while (!isDone) {
				Random random = new Random(1958042723);

				try {
					// Check if thread can be quitted
					if (timeToTest <= 0)
						isDone = true;

					// Decrement test time
					--timeToTest;

					// Generate test data
					/*for (int i = 0; i < 8; i++)
						testData[i] += (Math.abs(random.nextInt()) % 500);*/
					testData = test.calculate();

					/*
					 * Simulation time calculation
					 */

					// Get remainder for seconds
					int seconds = (int) (timeToTest / 10) % 60;

					// Get minutes
					int minutes = (int) (timeToTest / 10 - seconds) / 60;

					/*
					 * Real time calculation
					 */

					// Get realTotalTime
					int realTotalTime = TestSystem.RUN_TIME * 4;

					// Get realCurrentTime
					int realCurrentTime = realTotalTime - timeToTest * 4 / 10;

					// Get real hours
					int realMinutes = (11 * 60 + realCurrentTime) % 60;

					// Get real minutes
					int realHours = (11 * 60 + realCurrentTime - realMinutes) / 60;

					/*
					 * Time format and output
					 */

					// Get formatter
					java.text.DecimalFormat formatter = new java.text.DecimalFormat("00");

					// Update label
					progressMessage =
						"Uhrzeit:  "
							+ formatter.format(realHours)
							+ ":"
							+ formatter.format(realMinutes)
							+ " Uhr"
							+ "  /  Verbleibende Testzeit:  "
							+ formatter.format(minutes)
							+ " min   "
							+ formatter.format(seconds)
							+ " s";

					/*
					 *	Splash screens for different people 
					 */
					 
					// Workers
					if (timeToTest == 1350) showSplash(TestSystem.splashWorkers);
					
					// "Kaffeetanten"
					else if (timeToTest == 1050) showSplash(TestSystem.splashCoffee);
					
					// Young people
					else if (timeToTest == 600) showSplash(TestSystem.splashYoungPeople);

					// Sleep x milliseconds, see TestSystem.WAITING_TIME
					Thread.sleep(TestSystem.WAITING_TIME);

				} catch (Exception e) {
					// Quit loop if exception occured
					break;
				}
			}

			testIsRunning = false;
		}
	}

}