package sopra;

import java.applet.Applet;
import java.applet.AudioClip;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.net.MalformedURLException;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.SwingConstants;
import javax.swing.Timer;
import javax.swing.border.EmptyBorder;
import javax.swing.border.TitledBorder;
import java.util.Arrays;

/**
 * Main testing class
 * 
 * @author	Reto Geiger, Matthias Taugwalder
 */
public class TestSystem extends JFrame implements ActionListener {
	/**
	 * Constants
	 */
	private final String INITIAL_SCORE_VALUE = "0.00 ";
	private final int BAR_WIDTH = 400;
	static final int WAITING_TIME = 100; // 0.5 seconds	[in ms/10]
	static final int RUN_TIME = 150; // 2.5 minutes	[in seconds]
	static final int BORDER = 1000;

	/**
	 * Images (stored as icons)
	 */
	static ImageIcon icon = createImageIcon("/tasse.jpg");
	static String splashWorkers = "/arbeiter.jpg";
	static String splashCoffee = "/kaffeetanten.jpg";
	static String splashYoungPeople = "/jungeleute.jpg";

	/**
	 * sound
	 */
	static File soundApplause = new File("applause.wav");
	static File soundHorn = new File("airhorn.wav");

	/**
	 * TestRunner
	 */
	private TestRunner testRunner;

	/**
	 * Test data
	 */
	static float[] testData = new float[8];

	/**
	 * Global fields for name, score and bar
	 */
	private JPanel groupPanes[] = new JPanel[8];
	private String groupNames[];
	private JLabel groupLabels[] = new JLabel[8];
	private JTextField groupScores[] = new JTextField[8];
	private JPanel groupBars[] = new JPanel[8];

	/**
	 * last biggest element
	 */
	int lastBiggestElement = -1;

	/** Creates new form TestSystem */
	public TestSystem(String[] groupNames) {
		// Set groupNames
		this.groupNames = groupNames;

		// Initialize components
		initComponents();

		// Set remaining time
		updateRemainingTime(TestSystem.RUN_TIME);
	}

	/**
	 * Update remaining time label
	 * @param timeInSeconds
	 */
	private void updateRemainingTime(int timeInSeconds) {
		// Get remainder for seconds
		int seconds = timeInSeconds % 60;

		// Get minutes
		int minutes = (int) (timeInSeconds - seconds) / 60;

		// Get formatter
		java.text.DecimalFormat formatter = new java.text.DecimalFormat("00");

		// Update label
		remainingTimeLabel.setText(
			"Uhrzeit:  11:00 Uhr"
				+ "  /  Verbleibende Testzeit:  "
				+ formatter.format(minutes)
				+ " min   "
				+ formatter.format(seconds)
				+ " s");
	}

	/**
	 * Event handler for startButton click
	 * @param evt
	 */
	private void startButtonMouseClicked(java.awt.event.MouseEvent evt) {
		// Disable start button
		startButton.setEnabled(false);
		startButton.updateUI();

		// Initialize testRunner
		testRunner = new TestRunner();

		//Create a timer.
		Timer timer = new Timer(TestSystem.WAITING_TIME, new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				if (testRunner.isRunning()) {
					if (testRunner.isDone()) {
						// Enable start button
						startButton.setEnabled(true);
						startButton.updateUI();

						try {
							// Play audio notification
							AudioClip clip = Applet.newAudioClip(soundApplause.toURL());
							clip.play();

						} catch (MalformedURLException e) {
						}

						// Set ranks
						setRanks(TestSystem.testData);
						
						// Show winner
						new WinnerDialog(new javax.swing.JFrame(), true, groupNames[lastBiggestElement]).show();

						// Stop testRunner
						testRunner.stop();

					} else {
						// Set message
						remainingTimeLabel.setText(testRunner.getMessage());

						// Get test data	
						TestSystem.testData = testRunner.getTestData();

						// Get test data
						updateScores(TestSystem.testData);
					}
				}
			}
		});

		// Start test
		testRunner.go();

		// Start timer
		timer.start();
	}

	/**
	 * Sets the ranks for all groups
	 * @param scores Test data
	 */
	private void setRanks(float[] scores) {
		// Initialize new sortItem array
		SortItem sortItems[] = new SortItem[8];

		// Traverse group names
		for (int i = 0; i < groupNames.length; i++) {
			// Initialize sortItem
			sortItems[i] = new SortItem();
			
			// Set fields
			sortItems[i].Ind = i;
			sortItems[i].Name = groupNames[i];
			sortItems[i].Score = scores[i];
		}

		// Get array sorted
		Arrays.sort(sortItems, new Compare());

		// Initialize field for rank
		int rank = 1;

		// Traverse sorted array
		for (int j = 0; j < sortItems.length; j++) {
			// Get current item
			SortItem currentItem = sortItems[j];

			// Check if current element is real smaller than previous
			if(j > 0 && currentItem.Score < sortItems[j-1].Score) {
				// Set rank
				rank = j+1;
			}
			
			// Set text
			groupLabels[currentItem.Ind].setText("(" + rank + ")  " + groupNames[currentItem.Ind]);
		}
	}

	/**
	 * Set scores
	 * @param scores
	 */
	private void updateScores(float[] scores) {
		// Get biggest array element
		float biggestElement = maxElement(scores);

		// Calculate pixels per 1 score point
		float pixelPerScore = this.BAR_WIDTH / biggestElement;

		// Traverse fields
		for (int i = 0; i < 8; i++) {
			// Update score fields
			groupScores[i].setText(formatScoresOutput(scores[i]) + " ");

			// Calculate scoreBar width
			int scoreBarWidth = (int) (pixelPerScore * scores[i]);

			// Update score bars
			groupBars[i].setPreferredSize(new java.awt.Dimension(scoreBarWidth, 12));
			groupBars[i].updateUI();

			// Check if current element is the biggest element
			if (scores[i] == biggestElement) {
				// Reset last biggestElement
				if (lastBiggestElement > -1) {
					groupPanes[lastBiggestElement].setBorder(
						new javax.swing.border.LineBorder(new java.awt.Color(255, 255, 255), 2));
				}

				// Check if current biggestElement is unique
				//		case 1: 0 < i < 8, scores[i] > scores[i-1]
				//		case 2: i = 0, scores[0] > scores[1]
				if( (i > 0 && i < scores.length && scores[i] != scores[i-1]) || (i == 0 && scores[0] != scores[1])) {
					// Set new background
					groupPanes[i].setBorder(new javax.swing.border.LineBorder(new java.awt.Color(153, 0, 0), 2));

					// Store current element
					this.lastBiggestElement = i;
				} 
			}
		}
	}

	/**
	 * Format the float values for currency-like output
	 * @param score Float value
	 * @return Formatted String value
	 */
	public String formatScoresOutput(float score) {
		// Get formatter
		java.text.DecimalFormat formatter = new java.text.DecimalFormat("#,##0.00");

		// Get remainder
		float remainder = (score % 1);

		// C
		if ((remainder * 100 % 5) > 2.5f) {
			// Round up
			score = score + (5 - (remainder * 100 % 5) / 100);

		} else {
			// Round down
			score = score - (remainder * 100 % 5) / 100;
		}

		// Return formatted value
		return formatter.format(score);
	}

	/**
	 * Returns the biggest element of an array
	 * @param array
	 * @return
	 */
	private float maxElement(float[] array) {
		// Initialize maxElement
		float maxElement = 0;

		// Traverse array, store elements
		for (int i = 0; i < array.length; i++) {
			if (array[i] > maxElement)
				maxElement = array[i];
		}

		return maxElement;
	}

	/** 
	 * Initialize components
	 */
	private void initComponents() {
		// Initialize fields
		for (int i = 0; i < 8; i++) {
			groupLabels[i] = new JLabel();
			groupScores[i] = new JTextField();
			groupBars[i] = new JPanel();
			groupPanes[i] = new JPanel();
		}

		java.awt.GridBagConstraints gridBagConstraints;

		/*
		 * Initialize components
		 */
		windowPane = new JPanel();
		mainPane = new JPanel();
		contentPane = new JPanel();
		contentsPane = new JPanel();

		descBarPane = new JPanel();
		descPane = new JPanel();
		descGroup = new JLabel();
		descScore = new JLabel();
		descBar = new JLabel();

		group1BarPane = new JPanel();
		group2BarPane = new JPanel();
		group3BarPane = new JPanel();
		group4BarPane = new JPanel();
		group5BarPane = new JPanel();
		group6BarPane = new JPanel();
		group7BarPane = new JPanel();
		group8BarPane = new JPanel();

		controlPane = new JPanel();
		startButtonPane = new JPanel();
		remainingTimeLabel = new JLabel();
		startButton = new JButton();

		setTitle(" Sopra 2004 - Test Preisberechnung");
		setBackground(new java.awt.Color(255, 255, 255));
		addWindowListener(new java.awt.event.WindowAdapter() {
			public void windowClosing(java.awt.event.WindowEvent evt) {
				exitForm(evt);
			}
		});

		windowPane.setLayout(new java.awt.GridBagLayout());
		windowPane.setBackground(new java.awt.Color(255, 255, 255));

		mainPane.setLayout(new java.awt.BorderLayout());
		mainPane.setBackground(new java.awt.Color(255, 255, 255));
		mainPane.setBorder(new EmptyBorder(new java.awt.Insets(10, 10, 10, 10)));

		contentPane.setLayout(new java.awt.BorderLayout());
		contentPane.setBackground(new java.awt.Color(255, 255, 255));
		contentPane.setBorder(
			new TitledBorder(
				null,
				"  Test Preisberechnung  ",
				TitledBorder.DEFAULT_JUSTIFICATION,
				TitledBorder.DEFAULT_POSITION,
				new java.awt.Font("Dialog", 1, 14)));

		contentsPane.setLayout(new java.awt.GridBagLayout());
		contentsPane.setBackground(new java.awt.Color(255, 255, 255));

		/*
		 * Description
		 */
		descPane.setLayout(new java.awt.GridBagLayout());
		descPane.setBackground(new java.awt.Color(255, 255, 255));
		descPane.setBorder(new javax.swing.border.LineBorder(new java.awt.Color(233, 233, 233), 2));
		descPane.setMinimumSize(new java.awt.Dimension(725, 30));
		descPane.setPreferredSize(new java.awt.Dimension(725, 30));

		// GroupNames
		descGroup.setFont(new java.awt.Font("Dialog", 1, 12));
		descGroup.setHorizontalAlignment(SwingConstants.LEFT);
		descGroup.setText("G R U P P E");
		descGroup.setForeground(new java.awt.Color(66, 66, 66));
		descGroup.setMaximumSize(new java.awt.Dimension(150, 16));
		descGroup.setMinimumSize(new java.awt.Dimension(150, 16));
		descGroup.setPreferredSize(new java.awt.Dimension(150, 16));
		gridBagConstraints = new java.awt.GridBagConstraints();
		gridBagConstraints.insets = new java.awt.Insets(10, 0, 10, 10);
		descPane.add(descGroup, gridBagConstraints);

		// GroupScores
		descScore.setFont(new java.awt.Font("Dialog", 1, 12));
		descScore.setHorizontalAlignment(JTextField.CENTER);
		descScore.setText("G E W I N N");
		descScore.setForeground(new java.awt.Color(66, 66, 66));
		descScore.setFocusable(false);
		descScore.setMinimumSize(new java.awt.Dimension(100, 16));
		descScore.setPreferredSize(new java.awt.Dimension(100, 16));
		gridBagConstraints = new java.awt.GridBagConstraints();
		gridBagConstraints.insets = new java.awt.Insets(10, 10, 10, 10);
		descPane.add(descScore, gridBagConstraints);

		// GroupBarPane
		descBarPane.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.LEFT, 4, 4));
		descBarPane.setBackground(new java.awt.Color(255, 255, 255));
		descBarPane.setMinimumSize(new java.awt.Dimension(400, 16));
		descBarPane.setPreferredSize(new java.awt.Dimension(400, 16));

		// GroupBar
		descBar.setMinimumSize(new java.awt.Dimension(2, 12));
		descBar.setPreferredSize(new java.awt.Dimension(2, 12));
		descBarPane.add(descBar);
		gridBagConstraints = new java.awt.GridBagConstraints();
		gridBagConstraints.insets = new java.awt.Insets(0, 10, 0, 0);
		descPane.add(descBarPane, gridBagConstraints);

		// Group_Pane
		gridBagConstraints = new java.awt.GridBagConstraints();
		gridBagConstraints.gridx = 0;
		gridBagConstraints.gridy = 0;
		gridBagConstraints.ipadx = 10;
		gridBagConstraints.ipady = 0;
		gridBagConstraints.insets = new java.awt.Insets(0, 0, 0, 0);
		contentsPane.add(descPane, gridBagConstraints);

		/*
		 * Group 1
		 */
		groupPanes[0].setLayout(new java.awt.GridBagLayout());
		groupPanes[0].setBackground(new java.awt.Color(243, 243, 243));
		groupPanes[0].setBorder(new javax.swing.border.LineBorder(new java.awt.Color(255, 255, 255), 2));
		groupPanes[0].setMinimumSize(new java.awt.Dimension(725, 30));
		groupPanes[0].setPreferredSize(new java.awt.Dimension(725, 30));

		// GroupNames
		groupLabels[0].setFont(new java.awt.Font("Dialog", 1, 14));
		groupLabels[0].setHorizontalAlignment(SwingConstants.LEFT);
		groupLabels[0].setText(groupNames[0]);
		groupLabels[0].setMaximumSize(new java.awt.Dimension(150, 16));
		groupLabels[0].setMinimumSize(new java.awt.Dimension(150, 16));
		groupLabels[0].setPreferredSize(new java.awt.Dimension(150, 16));
		gridBagConstraints = new java.awt.GridBagConstraints();
		gridBagConstraints.insets = new java.awt.Insets(10, 10, 10, 10);
		groupPanes[0].add(groupLabels[0], gridBagConstraints);

		// GroupScores
		groupScores[0].setFont(new java.awt.Font("Dialog", 1, 18));
		groupScores[0].setHorizontalAlignment(JTextField.RIGHT);
		groupScores[0].setText(INITIAL_SCORE_VALUE);
		groupScores[0].setFocusable(false);
		groupScores[0].setMinimumSize(new java.awt.Dimension(100, 28));
		groupScores[0].setPreferredSize(new java.awt.Dimension(100, 28));
		gridBagConstraints = new java.awt.GridBagConstraints();
		gridBagConstraints.insets = new java.awt.Insets(10, 10, 10, 10);
		groupPanes[0].add(groupScores[0], gridBagConstraints);

		// GroupBarPane
		group1BarPane.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.LEFT, 4, 4));
		group1BarPane.setBackground(new java.awt.Color(243, 243, 243));
		group1BarPane.setMinimumSize(new java.awt.Dimension(400, 20));
		group1BarPane.setPreferredSize(new java.awt.Dimension(400, 20));

		// GroupBar
		groupBars[0].setMinimumSize(new java.awt.Dimension(2, 12));
		groupBars[0].setPreferredSize(new java.awt.Dimension(2, 12));
		group1BarPane.add(groupBars[0]);
		gridBagConstraints = new java.awt.GridBagConstraints();
		gridBagConstraints.insets = new java.awt.Insets(0, 20, 0, 0);
		groupPanes[0].add(group1BarPane, gridBagConstraints);

		// Group_Pane
		gridBagConstraints = new java.awt.GridBagConstraints();
		gridBagConstraints.gridx = 0;
		gridBagConstraints.gridy = 1;
		gridBagConstraints.ipadx = 10;
		gridBagConstraints.ipady = 10;
		gridBagConstraints.insets = new java.awt.Insets(10, 10, 10, 10);
		contentsPane.add(groupPanes[0], gridBagConstraints);

		/*
		 * Group 2
		 */
		groupPanes[1].setLayout(new java.awt.GridBagLayout());
		groupPanes[1].setBackground(new java.awt.Color(243, 243, 243));
		groupPanes[1].setBorder(new javax.swing.border.LineBorder(new java.awt.Color(255, 255, 255), 2));
		groupPanes[1].setMinimumSize(new java.awt.Dimension(725, 30));
		groupPanes[1].setPreferredSize(new java.awt.Dimension(725, 30));

		// GroupName
		groupLabels[1].setFont(new java.awt.Font("Dialog", 1, 14));
		groupLabels[1].setHorizontalAlignment(SwingConstants.LEFT);
		groupLabels[1].setText(groupNames[1]);
		groupLabels[1].setMaximumSize(new java.awt.Dimension(150, 16));
		groupLabels[1].setMinimumSize(new java.awt.Dimension(150, 16));
		groupLabels[1].setPreferredSize(new java.awt.Dimension(150, 16));
		gridBagConstraints = new java.awt.GridBagConstraints();
		gridBagConstraints.insets = new java.awt.Insets(10, 10, 10, 10);
		groupPanes[1].add(groupLabels[1], gridBagConstraints);

		// GroupScore
		groupScores[1].setFont(new java.awt.Font("Dialog", 1, 18));
		groupScores[1].setHorizontalAlignment(JTextField.RIGHT);
		groupScores[1].setText(INITIAL_SCORE_VALUE);
		groupScores[1].setFocusable(false);
		groupScores[1].setMinimumSize(new java.awt.Dimension(100, 28));
		groupScores[1].setPreferredSize(new java.awt.Dimension(100, 28));
		gridBagConstraints = new java.awt.GridBagConstraints();
		gridBagConstraints.insets = new java.awt.Insets(10, 10, 10, 10);
		groupPanes[1].add(groupScores[1], gridBagConstraints);

		// GroupBarPane
		group2BarPane.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.LEFT));
		group2BarPane.setBackground(new java.awt.Color(243, 243, 243));
		group2BarPane.setMinimumSize(new java.awt.Dimension(400, 20));
		group2BarPane.setPreferredSize(new java.awt.Dimension(400, 20));

		// GroupBar
		groupBars[1].setMinimumSize(new java.awt.Dimension(2, 12));
		groupBars[1].setPreferredSize(new java.awt.Dimension(2, 12));
		group2BarPane.add(groupBars[1]);
		gridBagConstraints = new java.awt.GridBagConstraints();
		gridBagConstraints.insets = new java.awt.Insets(0, 20, 0, 0);
		groupPanes[1].add(group2BarPane, gridBagConstraints);

		// Group_Pane
		gridBagConstraints = new java.awt.GridBagConstraints();
		gridBagConstraints.gridx = 0;
		gridBagConstraints.gridy = 2;
		gridBagConstraints.ipadx = 10;
		gridBagConstraints.ipady = 10;
		gridBagConstraints.insets = new java.awt.Insets(10, 10, 10, 10);
		contentsPane.add(groupPanes[1], gridBagConstraints);

		/**
		 * Group 3
		 */
		groupPanes[2].setLayout(new java.awt.GridBagLayout());
		groupPanes[2].setBackground(new java.awt.Color(243, 243, 243));
		groupPanes[2].setBorder(new javax.swing.border.LineBorder(new java.awt.Color(255, 255, 255), 2));
		groupPanes[2].setMinimumSize(new java.awt.Dimension(725, 30));
		groupPanes[2].setPreferredSize(new java.awt.Dimension(725, 30));

		// GroupName
		groupLabels[2].setFont(new java.awt.Font("Dialog", 1, 14));
		groupLabels[2].setHorizontalAlignment(SwingConstants.LEFT);
		groupLabels[2].setText(groupNames[2]);
		groupLabels[2].setMaximumSize(new java.awt.Dimension(150, 16));
		groupLabels[2].setMinimumSize(new java.awt.Dimension(150, 16));
		groupLabels[2].setPreferredSize(new java.awt.Dimension(150, 16));
		gridBagConstraints = new java.awt.GridBagConstraints();
		gridBagConstraints.insets = new java.awt.Insets(10, 10, 10, 10);
		groupPanes[2].add(groupLabels[2], gridBagConstraints);

		// GroupScore
		groupScores[2].setFont(new java.awt.Font("Dialog", 1, 18));
		groupScores[2].setHorizontalAlignment(JTextField.RIGHT);
		groupScores[2].setText(INITIAL_SCORE_VALUE);
		groupScores[2].setFocusable(false);
		groupScores[2].setMinimumSize(new java.awt.Dimension(100, 28));
		groupScores[2].setPreferredSize(new java.awt.Dimension(100, 28));
		gridBagConstraints = new java.awt.GridBagConstraints();
		gridBagConstraints.insets = new java.awt.Insets(10, 10, 10, 10);
		groupPanes[2].add(groupScores[2], gridBagConstraints);

		// GroupBarPane
		group3BarPane.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.LEFT));
		group3BarPane.setBackground(new java.awt.Color(243, 243, 243));
		group3BarPane.setMinimumSize(new java.awt.Dimension(400, 20));
		group3BarPane.setPreferredSize(new java.awt.Dimension(400, 20));

		// GroupBar
		groupBars[2].setMinimumSize(new java.awt.Dimension(2, 12));
		groupBars[2].setPreferredSize(new java.awt.Dimension(2, 12));
		group3BarPane.add(groupBars[2]);
		gridBagConstraints = new java.awt.GridBagConstraints();
		gridBagConstraints.insets = new java.awt.Insets(0, 20, 0, 0);
		groupPanes[2].add(group3BarPane, gridBagConstraints);

		// Group_Pane
		gridBagConstraints = new java.awt.GridBagConstraints();
		gridBagConstraints.gridx = 0;
		gridBagConstraints.gridy = 3;
		gridBagConstraints.ipadx = 10;
		gridBagConstraints.ipady = 10;
		gridBagConstraints.insets = new java.awt.Insets(10, 10, 10, 10);
		contentsPane.add(groupPanes[2], gridBagConstraints);

		/**
		 * Group 4
		 */
		groupPanes[3].setLayout(new java.awt.GridBagLayout());
		groupPanes[3].setBackground(new java.awt.Color(243, 243, 243));
		groupPanes[3].setBorder(new javax.swing.border.LineBorder(new java.awt.Color(255, 255, 255), 2));
		groupPanes[3].setMinimumSize(new java.awt.Dimension(725, 30));
		groupPanes[3].setPreferredSize(new java.awt.Dimension(725, 30));

		// GroupName
		groupLabels[3].setFont(new java.awt.Font("Dialog", 1, 14));
		groupLabels[3].setHorizontalAlignment(SwingConstants.LEFT);
		groupLabels[3].setText(groupNames[3]);
		groupLabels[3].setMaximumSize(new java.awt.Dimension(150, 16));
		groupLabels[3].setMinimumSize(new java.awt.Dimension(150, 16));
		groupLabels[3].setPreferredSize(new java.awt.Dimension(150, 16));
		gridBagConstraints = new java.awt.GridBagConstraints();
		gridBagConstraints.insets = new java.awt.Insets(10, 10, 10, 10);
		groupPanes[3].add(groupLabels[3], gridBagConstraints);

		// GroupScore
		groupScores[3].setFont(new java.awt.Font("Dialog", 1, 18));
		groupScores[3].setHorizontalAlignment(JTextField.RIGHT);
		groupScores[3].setText(INITIAL_SCORE_VALUE);
		groupScores[3].setFocusable(false);
		groupScores[3].setMinimumSize(new java.awt.Dimension(100, 28));
		groupScores[3].setPreferredSize(new java.awt.Dimension(100, 28));
		gridBagConstraints = new java.awt.GridBagConstraints();
		gridBagConstraints.insets = new java.awt.Insets(10, 10, 10, 10);
		groupPanes[3].add(groupScores[3], gridBagConstraints);

		// GroupBarPane
		group4BarPane.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.LEFT));
		group4BarPane.setBackground(new java.awt.Color(243, 243, 243));
		group4BarPane.setMinimumSize(new java.awt.Dimension(400, 20));
		group4BarPane.setPreferredSize(new java.awt.Dimension(400, 20));

		// GroupBar
		groupBars[3].setMinimumSize(new java.awt.Dimension(2, 12));
		groupBars[3].setPreferredSize(new java.awt.Dimension(2, 12));
		group4BarPane.add(groupBars[3]);
		gridBagConstraints = new java.awt.GridBagConstraints();
		gridBagConstraints.insets = new java.awt.Insets(0, 20, 0, 0);
		groupPanes[3].add(group4BarPane, gridBagConstraints);

		// Group_Pane
		gridBagConstraints = new java.awt.GridBagConstraints();
		gridBagConstraints.gridx = 0;
		gridBagConstraints.gridy = 4;
		gridBagConstraints.ipadx = 10;
		gridBagConstraints.ipady = 10;
		gridBagConstraints.insets = new java.awt.Insets(10, 10, 10, 10);
		contentsPane.add(groupPanes[3], gridBagConstraints);

		/*
		 * Group 5
		 */
		groupPanes[4].setLayout(new java.awt.GridBagLayout());
		groupPanes[4].setBackground(new java.awt.Color(243, 243, 243));
		groupPanes[4].setBorder(new javax.swing.border.LineBorder(new java.awt.Color(255, 255, 255), 2));
		groupPanes[4].setMinimumSize(new java.awt.Dimension(725, 30));
		groupPanes[4].setPreferredSize(new java.awt.Dimension(725, 30));

		// GroupName
		groupLabels[4].setFont(new java.awt.Font("Dialog", 1, 14));
		groupLabels[4].setHorizontalAlignment(SwingConstants.LEFT);
		groupLabels[4].setText(groupNames[4]);
		groupLabels[4].setMaximumSize(new java.awt.Dimension(150, 16));
		groupLabels[4].setMinimumSize(new java.awt.Dimension(150, 16));
		groupLabels[4].setPreferredSize(new java.awt.Dimension(150, 16));
		gridBagConstraints = new java.awt.GridBagConstraints();
		gridBagConstraints.insets = new java.awt.Insets(10, 10, 10, 10);
		groupPanes[4].add(groupLabels[4], gridBagConstraints);

		// GroupScore
		groupScores[4].setFont(new java.awt.Font("Dialog", 1, 18));
		groupScores[4].setHorizontalAlignment(JTextField.RIGHT);
		groupScores[4].setText(INITIAL_SCORE_VALUE);
		groupScores[4].setFocusable(false);
		groupScores[4].setMinimumSize(new java.awt.Dimension(100, 28));
		groupScores[4].setPreferredSize(new java.awt.Dimension(100, 28));
		gridBagConstraints = new java.awt.GridBagConstraints();
		gridBagConstraints.insets = new java.awt.Insets(10, 10, 10, 10);
		groupPanes[4].add(groupScores[4], gridBagConstraints);

		// GroupBarPane
		group5BarPane.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.LEFT));
		group5BarPane.setBackground(new java.awt.Color(243, 243, 243));
		group5BarPane.setMinimumSize(new java.awt.Dimension(400, 20));
		group5BarPane.setPreferredSize(new java.awt.Dimension(400, 20));

		// GroupBar
		groupBars[4].setMinimumSize(new java.awt.Dimension(2, 12));
		groupBars[4].setPreferredSize(new java.awt.Dimension(2, 12));
		group5BarPane.add(groupBars[4]);
		gridBagConstraints = new java.awt.GridBagConstraints();
		gridBagConstraints.insets = new java.awt.Insets(0, 20, 0, 0);
		groupPanes[4].add(group5BarPane, gridBagConstraints);

		// Group_Pane
		gridBagConstraints = new java.awt.GridBagConstraints();
		gridBagConstraints.gridx = 0;
		gridBagConstraints.gridy = 5;
		gridBagConstraints.ipadx = 10;
		gridBagConstraints.ipady = 10;
		gridBagConstraints.insets = new java.awt.Insets(10, 10, 10, 10);
		contentsPane.add(groupPanes[4], gridBagConstraints);

		/**
		 * Group 6
		 */
		groupPanes[5].setLayout(new java.awt.GridBagLayout());
		groupPanes[5].setBackground(new java.awt.Color(243, 243, 243));
		groupPanes[5].setBorder(new javax.swing.border.LineBorder(new java.awt.Color(255, 255, 255), 2));
		groupPanes[5].setMinimumSize(new java.awt.Dimension(725, 30));
		groupPanes[5].setPreferredSize(new java.awt.Dimension(725, 30));

		// GroupName
		groupLabels[5].setFont(new java.awt.Font("Dialog", 1, 14));
		groupLabels[5].setHorizontalAlignment(SwingConstants.LEFT);
		groupLabels[5].setText(groupNames[5]);
		groupLabels[5].setMaximumSize(new java.awt.Dimension(150, 16));
		groupLabels[5].setMinimumSize(new java.awt.Dimension(150, 16));
		groupLabels[5].setPreferredSize(new java.awt.Dimension(150, 16));
		gridBagConstraints = new java.awt.GridBagConstraints();
		gridBagConstraints.insets = new java.awt.Insets(10, 10, 10, 10);
		groupPanes[5].add(groupLabels[5], gridBagConstraints);

		// GroupScore
		groupScores[5].setFont(new java.awt.Font("Dialog", 1, 18));
		groupScores[5].setHorizontalAlignment(JTextField.RIGHT);
		groupScores[5].setText(INITIAL_SCORE_VALUE);
		groupScores[5].setFocusable(false);
		groupScores[5].setMinimumSize(new java.awt.Dimension(100, 28));
		groupScores[5].setPreferredSize(new java.awt.Dimension(100, 28));
		gridBagConstraints = new java.awt.GridBagConstraints();
		gridBagConstraints.insets = new java.awt.Insets(10, 10, 10, 10);
		groupPanes[5].add(groupScores[5], gridBagConstraints);

		// GroupBarPane
		group6BarPane.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.LEFT));
		group6BarPane.setBackground(new java.awt.Color(243, 243, 243));
		group6BarPane.setMinimumSize(new java.awt.Dimension(400, 20));
		group6BarPane.setPreferredSize(new java.awt.Dimension(400, 20));

		// GroupBar
		groupBars[5].setMinimumSize(new java.awt.Dimension(2, 12));
		groupBars[5].setPreferredSize(new java.awt.Dimension(2, 12));
		group6BarPane.add(groupBars[5]);
		gridBagConstraints = new java.awt.GridBagConstraints();
		gridBagConstraints.insets = new java.awt.Insets(0, 20, 0, 0);
		groupPanes[5].add(group6BarPane, gridBagConstraints);

		// Group_Pane
		gridBagConstraints = new java.awt.GridBagConstraints();
		gridBagConstraints.gridx = 0;
		gridBagConstraints.gridy = 6;
		gridBagConstraints.ipadx = 10;
		gridBagConstraints.ipady = 10;
		gridBagConstraints.insets = new java.awt.Insets(10, 10, 10, 10);
		contentsPane.add(groupPanes[5], gridBagConstraints);

		/*
		 * Group 7
		 */
		groupPanes[6].setLayout(new java.awt.GridBagLayout());
		groupPanes[6].setBackground(new java.awt.Color(243, 243, 243));
		groupPanes[6].setBorder(new javax.swing.border.LineBorder(new java.awt.Color(255, 255, 255), 2));
		groupPanes[6].setMinimumSize(new java.awt.Dimension(725, 30));
		groupPanes[6].setPreferredSize(new java.awt.Dimension(725, 30));

		// GroupName
		groupLabels[6].setFont(new java.awt.Font("Dialog", 1, 14));
		groupLabels[6].setHorizontalAlignment(SwingConstants.LEFT);
		groupLabels[6].setText(groupNames[6]);
		groupLabels[6].setMaximumSize(new java.awt.Dimension(150, 16));
		groupLabels[6].setMinimumSize(new java.awt.Dimension(150, 16));
		groupLabels[6].setPreferredSize(new java.awt.Dimension(150, 16));
		gridBagConstraints = new java.awt.GridBagConstraints();
		gridBagConstraints.insets = new java.awt.Insets(10, 10, 10, 10);
		groupPanes[6].add(groupLabels[6], gridBagConstraints);

		// GroupScore
		groupScores[6].setFont(new java.awt.Font("Dialog", 1, 18));
		groupScores[6].setHorizontalAlignment(JTextField.RIGHT);
		groupScores[6].setText(INITIAL_SCORE_VALUE);
		groupScores[6].setFocusable(false);
		groupScores[6].setMinimumSize(new java.awt.Dimension(100, 28));
		groupScores[6].setPreferredSize(new java.awt.Dimension(100, 28));
		gridBagConstraints = new java.awt.GridBagConstraints();
		gridBagConstraints.insets = new java.awt.Insets(10, 10, 10, 10);
		groupPanes[6].add(groupScores[6], gridBagConstraints);

		// GroupBarPane
		group7BarPane.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.LEFT));
		group7BarPane.setBackground(new java.awt.Color(243, 243, 243));
		group7BarPane.setMinimumSize(new java.awt.Dimension(400, 20));
		group7BarPane.setPreferredSize(new java.awt.Dimension(400, 20));

		// GroupBar
		groupBars[6].setMinimumSize(new java.awt.Dimension(2, 12));
		groupBars[6].setPreferredSize(new java.awt.Dimension(2, 12));
		group7BarPane.add(groupBars[6]);
		gridBagConstraints = new java.awt.GridBagConstraints();
		gridBagConstraints.insets = new java.awt.Insets(0, 20, 0, 0);
		groupPanes[6].add(group7BarPane, gridBagConstraints);

		// Group_Pane
		gridBagConstraints = new java.awt.GridBagConstraints();
		gridBagConstraints.gridx = 0;
		gridBagConstraints.gridy = 7;
		gridBagConstraints.ipadx = 10;
		gridBagConstraints.ipady = 10;
		gridBagConstraints.insets = new java.awt.Insets(10, 10, 10, 10);
		contentsPane.add(groupPanes[6], gridBagConstraints);

		/**
		 * Group 8
		 */
		groupPanes[7].setLayout(new java.awt.GridBagLayout());
		groupPanes[7].setBackground(new java.awt.Color(243, 243, 243));
		groupPanes[7].setBorder(new javax.swing.border.LineBorder(new java.awt.Color(255, 255, 255), 2));
		groupPanes[7].setMinimumSize(new java.awt.Dimension(725, 30));
		groupPanes[7].setPreferredSize(new java.awt.Dimension(725, 30));

		// GroupName
		groupLabels[7].setFont(new java.awt.Font("Dialog", 1, 14));
		groupLabels[7].setHorizontalAlignment(SwingConstants.LEFT);
		groupLabels[7].setText(groupNames[7]);
		groupLabels[7].setMaximumSize(new java.awt.Dimension(150, 16));
		groupLabels[7].setMinimumSize(new java.awt.Dimension(150, 16));
		groupLabels[7].setPreferredSize(new java.awt.Dimension(150, 16));
		gridBagConstraints = new java.awt.GridBagConstraints();
		gridBagConstraints.insets = new java.awt.Insets(10, 10, 10, 10);
		groupPanes[7].add(groupLabels[7], gridBagConstraints);

		// GroupScore
		groupScores[7].setFont(new java.awt.Font("Dialog", 1, 18));
		groupScores[7].setHorizontalAlignment(JTextField.RIGHT);
		groupScores[7].setText(INITIAL_SCORE_VALUE);
		groupScores[7].setFocusable(false);
		groupScores[7].setMinimumSize(new java.awt.Dimension(100, 28));
		groupScores[7].setPreferredSize(new java.awt.Dimension(100, 28));
		gridBagConstraints = new java.awt.GridBagConstraints();
		gridBagConstraints.insets = new java.awt.Insets(10, 10, 10, 10);
		groupPanes[7].add(groupScores[7], gridBagConstraints);

		// GroupBarPane
		group8BarPane.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.LEFT));
		group8BarPane.setBackground(new java.awt.Color(243, 243, 243));
		group8BarPane.setMinimumSize(new java.awt.Dimension(400, 20));
		group8BarPane.setPreferredSize(new java.awt.Dimension(400, 20));

		// GroupBar
		groupBars[7].setMinimumSize(new java.awt.Dimension(2, 12));
		groupBars[7].setPreferredSize(new java.awt.Dimension(2, 12));
		group8BarPane.add(groupBars[7]);
		gridBagConstraints = new java.awt.GridBagConstraints();
		gridBagConstraints.insets = new java.awt.Insets(0, 20, 0, 0);
		groupPanes[7].add(group8BarPane, gridBagConstraints);

		// Group_Pane
		gridBagConstraints = new java.awt.GridBagConstraints();
		gridBagConstraints.gridx = 0;
		gridBagConstraints.gridy = 8;
		gridBagConstraints.ipadx = 10;
		gridBagConstraints.ipady = 10;
		gridBagConstraints.insets = new java.awt.Insets(10, 10, 10, 10);
		contentsPane.add(groupPanes[7], gridBagConstraints);

		/**
		 * Other components
		 */
		contentPane.add(contentsPane, java.awt.BorderLayout.NORTH);
		controlPane.setLayout(new java.awt.GridBagLayout());

		controlPane.setBackground(new java.awt.Color(255, 255, 255));
		controlPane.setMinimumSize(new java.awt.Dimension(100, 100));
		startButtonPane.setLayout(new java.awt.GridBagLayout());
		startButtonPane.setBackground(new java.awt.Color(255, 255, 255));
		startButtonPane.setMinimumSize(new java.awt.Dimension(750, 46));
		startButtonPane.setPreferredSize(new java.awt.Dimension(750, 46));

		remainingTimeLabel.setFont(new java.awt.Font("Dialog", 1, 14));
		remainingTimeLabel.setHorizontalAlignment(SwingConstants.LEFT);
		remainingTimeLabel.setMinimumSize(new java.awt.Dimension(580, 19));
		remainingTimeLabel.setPreferredSize(new java.awt.Dimension(580, 19));
		gridBagConstraints = new java.awt.GridBagConstraints();
		gridBagConstraints.insets = new java.awt.Insets(10, 10, 10, 10);
		startButtonPane.add(remainingTimeLabel, gridBagConstraints);

		startButton.setText("Test starten");
		startButton.addMouseListener(new java.awt.event.MouseAdapter() {
			public void mouseClicked(java.awt.event.MouseEvent evt) {
				startButtonMouseClicked(evt);
			}
		});

		gridBagConstraints = new java.awt.GridBagConstraints();
		gridBagConstraints.insets = new java.awt.Insets(10, 10, 10, 10);
		gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
		startButtonPane.add(startButton, gridBagConstraints);

		controlPane.add(startButtonPane, new java.awt.GridBagConstraints());
		contentPane.add(controlPane, java.awt.BorderLayout.SOUTH);
		mainPane.add(contentPane, java.awt.BorderLayout.CENTER);
		windowPane.add(mainPane, new java.awt.GridBagConstraints());
		getContentPane().add(windowPane, java.awt.BorderLayout.CENTER);

		// Get screen size, center window on Screen
		java.awt.Dimension screenSize = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
		//setBounds((screenSize.width - 800) / 2, (screenSize.height - 600) / 2, 800, 600);
		setBounds(0, 0, screenSize.width, screenSize.height);
	}

	/**
	 * Exit the Application
	 */
	private void exitForm(java.awt.event.WindowEvent evt) {
		System.exit(0);
	}

	/**
	 * @param args the command line arguments
	 */
	public static void main(String args[]) {
		// Intialize groupNames
		String groupNames[] = new String[8];

		// Set names
		groupNames[0] = "RIRO-Soft";
		groupNames[1] = "bQm (DISQUAL)";
		groupNames[2] = "A-Team";
		groupNames[3] = "Moloch";
		groupNames[4] = "RPG";
		groupNames[5] = "Coffee.com";
		groupNames[6] = "Phumlaz Inc.";
		groupNames[7] = "Sopla-null-viel";

		// Initialize new TestSystem object
		TestSystem testSystem = new TestSystem(groupNames);

		// Show window
		testSystem.show();
	}

	private JPanel contentPane;
	private JPanel contentsPane;
	private JPanel controlPane;

	private JPanel descBarPane;
	private JPanel descPane;
	private JLabel descGroup;
	private JLabel descScore;
	private JLabel descBar;

	private JPanel group1BarPane;
	private JPanel group1Pane;

	private JPanel group2BarPane;
	private JPanel group2Pane;

	private JPanel group3BarPane;
	private JPanel group3Pane;

	private JPanel group4BarPane;
	private JPanel group4Pane;

	private JPanel group5BarPane;
	private JPanel group5Pane;

	private JPanel group6BarPane;
	private JPanel group6Pane;

	private JPanel group7BarPane;
	private JPanel group7Pane;

	private JPanel group8BarPane;
	private JPanel group8Pane;

	private JLabel remainingTimeLabel;
	private JPanel mainPane;
	private JPanel windowPane;
	private JButton startButton;
	private JPanel startButtonPane;

	public void actionPerformed(ActionEvent e) {
		// Update label
		remainingTimeLabel.setText("Gugus");
	}

	protected static ImageIcon createImageIcon(String path) {
		java.net.URL imgURL = TestSystem.class.getResource(path);
		if (imgURL != null) {
			return new ImageIcon(imgURL);

		} else {
			System.err.println("Couldn't find file: " + path);
			return null;
		}
	}

	class SortItem {
		public int Ind;
		public String Name;
		public float Score;
	}

	class Compare implements java.util.Comparator {
		public int compare(Object o1, Object o2) {
			if (!(o1 instanceof SortItem))
				throw new ClassCastException();
			if (!(o2 instanceof SortItem))
				throw new ClassCastException();

			//Do an upper-case comparison
			if (((SortItem) o1).Score > ((SortItem) o2).Score)
				return -1;
			else if (((SortItem) o1).Score < ((SortItem) o2).Score)
				return 1;
			else
				return 0;
		}
	}
}
